@@ -70,9 +70,18 @@ function check() {
     local misplaced_end=$8
     local primary_start=${9:-}
     local primary_end=${10:-}
+    local check_setup=${11:-true}
 
-    local log=$(grep -l +backfilling $dir/osd.*.log)
-    test -n "$log" || return 1
+    local log=$(grep -l +backfilling $dir/osd.$primary.log)
+    if [ $check_setup = "true" ];
+    then
+      local alllogs=$(grep -l +backfilling $dir/osd.*.log)
+      if [ "$(echo "$alllogs" | wc -w)" != "1" ];
+      then
+        echo "Test setup failure, a single OSD should have performed backfill"
+        return 1
+      fi
+    fi
 
     local addp=" "
     if [ "$type" = "erasure" ];
@@ -340,8 +349,11 @@ function TEST_backfill_out2() {
     ceph osd pool set $poolname size 3
     ceph osd out osd.${otherosd}
     ceph osd out osd.${primary}
+    # Primary might change before backfill starts
+    sleep 2
+    primary=$(get_primary $poolname obj1)
     ceph osd unset nobackfill
-    ceph tell osd.$(get_primary $poolname obj1) debug kick_recovery_wq 0
+    ceph tell osd.$primary debug kick_recovery_wq 0
     sleep 2
 
     wait_for_clean || return 1
@@ -394,8 +406,11 @@ function TEST_backfill_sizeup4_allout() {
     ceph osd out osd.$otherosd
     ceph osd out osd.$primary
     ceph osd pool set $poolname size 4
+    # Primary might change before backfill starts
+    sleep 2
+    primary=$(get_primary $poolname obj1)
     ceph osd unset nobackfill
-    ceph tell osd.$(get_primary $poolname obj1) debug kick_recovery_wq 0
+    ceph tell osd.$primary debug kick_recovery_wq 0
     sleep 2
 
     wait_for_clean || return 1
@@ -453,15 +468,21 @@ function TEST_backfill_remapped() {
     ceph osd out osd.${primary}
     ceph osd pool set $poolname size 2
     sleep 2
+
+    # primary may change due to invalidating the old pg_temp, which was [1,2,0],
+    # but up_primary (3) chooses [0,1] for acting.
+    primary=$(get_primary $poolname obj1)
+
     ceph osd unset nobackfill
-    ceph tell osd.$(get_primary $poolname obj1) debug kick_recovery_wq 0
+    ceph tell osd.$primary debug kick_recovery_wq 0
+
     sleep 2
 
     wait_for_clean || return 1
 
     local misplaced=$(expr $objects \* 2)
 
-    check $dir $PG $primary replicated 0 0 $misplaced $objects || return 1
+    check $dir $PG $primary replicated 0 0 $misplaced $objects "" "" false || return 1
 
     delete_pool $poolname
     kill_daemons $dir || return 1
@@ -509,8 +530,11 @@ function TEST_backfill_ec_all_out() {
     do
         ceph osd out osd.$o
     done
+    # Primary might change before backfill starts
+    sleep 2
+    primary=$(get_primary $poolname obj1)
     ceph osd unset nobackfill
-    ceph tell osd.$(get_primary $poolname obj1) debug kick_recovery_wq 0
+    ceph tell osd.$primary debug kick_recovery_wq 0
     sleep 2
 
     wait_for_clean || return 1
@@ -556,8 +580,11 @@ function TEST_backfill_ec_prim_out() {
 
     ceph osd set nobackfill
     ceph osd out osd.$primary
+    # Primary might change before backfill starts
+    sleep 2
+    primary=$(get_primary $poolname obj1)
     ceph osd unset nobackfill
-    ceph tell osd.$(get_primary $poolname obj1) debug kick_recovery_wq 0
+    ceph tell osd.$primary debug kick_recovery_wq 0
     sleep 2
 
     wait_for_clean || return 1
@@ -611,8 +638,11 @@ function TEST_backfill_ec_down_all_out() {
     do
         ceph osd out osd.$o
     done
+    # Primary might change before backfill starts
+    sleep 2
+    primary=$(get_primary $poolname obj1)
     ceph osd unset nobackfill
-    ceph tell osd.$(get_primary $poolname obj1) debug kick_recovery_wq 0
+    ceph tell osd.$primary debug kick_recovery_wq 0
     sleep 2
     flush_pg_stats
 
@@ -622,7 +652,7 @@ function TEST_backfill_ec_down_all_out() {
     while(true)
     do
       if test "$(ceph --format json pg dump pgs |
-         jq '[.[] | .state | select(. == "incomplete")] | length')" -ne "0"
+         jq '.pg_stats | [.[] | .state | select(. == "incomplete")] | length')" -ne "0"
       then
         sleep 2
         continue
@@ -692,8 +722,11 @@ function TEST_backfill_ec_down_out() {
     kill $(cat $dir/osd.${otherosd}.pid)
     ceph osd down osd.${otherosd}
     ceph osd out osd.${otherosd}
+    # Primary might change before backfill starts
+    sleep 2
+    primary=$(get_primary $poolname obj1)
     ceph osd unset nobackfill
-    ceph tell osd.$(get_primary $poolname obj1) debug kick_recovery_wq 0
+    ceph tell osd.$primary debug kick_recovery_wq 0
     sleep 2
 
     wait_for_clean || return 1
