@@ -27,33 +27,47 @@
 #undef dout_prefix
 #define dout_prefix *_dout << "cephx server " << entity_name << ": "
 
-int CephxServiceHandler::start_session(EntityName& name, bufferlist::iterator& indata, bufferlist& result_bl, AuthCapsInfo& caps)
+int CephxServiceHandler::start_session(
+  const EntityName& name,
+  size_t connection_secret_required_length,
+  bufferlist *result_bl,
+  AuthCapsInfo *caps,
+  CryptoKey *session_key,
+  std::string *connection_secret)
 {
   entity_name = name;
 
   uint64_t min = 1; // always non-zero
   uint64_t max = std::numeric_limits<uint64_t>::max();
   server_challenge = ceph::util::generate_random_number<uint64_t>(min, max);
-  ldout(cct, 10) << "start_session server_challenge " << hex << server_challenge << dec << dendl;
+  ldout(cct, 10) << "start_session server_challenge "
+		 << hex << server_challenge << dec << dendl;
 
   CephXServerChallenge ch;
   ch.server_challenge = server_challenge;
-  encode(ch, result_bl);
-  return CEPH_AUTH_CEPHX;
+  encode(ch, *result_bl);
+  return 0;
 }
 
-int CephxServiceHandler::handle_request(bufferlist::iterator& indata, bufferlist& result_bl, uint64_t& global_id, AuthCapsInfo& caps, uint64_t *auid)
+int CephxServiceHandler::handle_request(
+  bufferlist::const_iterator& indata,
+  size_t connection_secret_required_len,
+  bufferlist *result_bl,
+  uint64_t *global_id,
+  AuthCapsInfo *caps,
+  CryptoKey *psession_key,
+  std::string *pconnection_secret)
 {
   int ret = 0;
 
   struct CephXRequestHeader cephx_header;
   decode(cephx_header, indata);
 
-
   switch (cephx_header.request_type) {
   case CEPHX_GET_AUTH_SESSION_KEY:
     {
-      ldout(cct, 10) << "handle_request get_auth_session_key for " << entity_name << dendl;
+      ldout(cct, 10) << "handle_request get_auth_session_key for "
+		     << entity_name << dendl;
 
       CephXAuthenticate req;
       decode(req, indata);
@@ -102,22 +116,26 @@ int CephxServiceHandler::handle_request(bufferlist::iterator& indata, bufferlist
 
       if (cephx_decode_ticket(cct, key_server, CEPH_ENTITY_TYPE_AUTH,
 			      req.old_ticket, old_ticket_info)) {
-        global_id = old_ticket_info.ticket.global_id;
-        ldout(cct, 10) << "decoded old_ticket with global_id=" << global_id << dendl;
+        *global_id = old_ticket_info.ticket.global_id;
+        ldout(cct, 10) << "decoded old_ticket with global_id=" << *global_id
+		       << dendl;
         should_enc_ticket = true;
       }
 
-      info.ticket.init_timestamps(ceph_clock_now(), cct->_conf->auth_mon_ticket_ttl);
+      ldout(cct,10) << __func__ << " auth ticket global_id " << *global_id
+		    << dendl;
+      info.ticket.init_timestamps(ceph_clock_now(),
+				  cct->_conf->auth_mon_ticket_ttl);
       info.ticket.name = entity_name;
-      info.ticket.global_id = global_id;
-      info.ticket.auid = eauth.auid;
+      info.ticket.global_id = *global_id;
       info.validity += cct->_conf->auth_mon_ticket_ttl;
 
-      if (auid) *auid = eauth.auid;
-
       key_server->generate_secret(session_key);
 
       info.session_key = session_key;
+      if (psession_key) {
+	*psession_key = session_key;
+      }
       info.service_id = CEPH_ENTITY_TYPE_AUTH;
       if (!key_server->get_service_secret(CEPH_ENTITY_TYPE_AUTH, info.service_secret, info.secret_id)) {
         ldout(cct, 0) << " could not get service secret for auth subsystem" << dendl;
@@ -128,21 +146,74 @@ int CephxServiceHandler::handle_request(bufferlist::iterator& indata, bufferlist
       vector<CephXSessionAuthInfo> info_vec;
       info_vec.push_back(info);
 
-      build_cephx_response_header(cephx_header.request_type, 0, result_bl);
-      if (!cephx_build_service_ticket_reply(cct, eauth.key, info_vec, should_enc_ticket,
-					    old_ticket_info.session_key, result_bl)) {
+      build_cephx_response_header(cephx_header.request_type, 0, *result_bl);
+      if (!cephx_build_service_ticket_reply(
+	    cct, eauth.key, info_vec, should_enc_ticket,
+	    old_ticket_info.session_key, *result_bl)) {
 	ret = -EIO;
+	break;
       }
 
-      if (!key_server->get_service_caps(entity_name, CEPH_ENTITY_TYPE_MON, caps)) {
+      if (!key_server->get_service_caps(entity_name, CEPH_ENTITY_TYPE_MON,
+					*caps)) {
         ldout(cct, 0) << " could not get mon caps for " << entity_name << dendl;
         ret = -EACCES;
+	break;
       } else {
-        char *caps_str = caps.caps.c_str();
+        char *caps_str = caps->caps.c_str();
         if (!caps_str || !caps_str[0]) {
           ldout(cct,0) << "mon caps null for " << entity_name << dendl;
           ret = -EACCES;
+	  break;
         }
+
+	if (req.other_keys) {
+	  // nautilus+ client
+	  // generate a connection_secret
+	  bufferlist cbl;
+	  if (pconnection_secret) {
+	    pconnection_secret->resize(connection_secret_required_len);
+	    if (connection_secret_required_len) {
+	      cct->random()->get_bytes(pconnection_secret->data(),
+				       connection_secret_required_len);
+	    }
+	    std::string err;
+	    if (encode_encrypt(cct, *pconnection_secret, session_key, cbl,
+			       err)) {
+	      lderr(cct) << __func__ << " failed to encrypt connection secret, "
+			 << err << dendl;
+	      ret = -EACCES;
+	      break;
+	    }
+	  }
+	  encode(cbl, *result_bl);
+	  // provite all of the other tickets at the same time
+	  vector<CephXSessionAuthInfo> info_vec;
+	  for (uint32_t service_id = 1; service_id <= req.other_keys;
+	       service_id <<= 1) {
+	    if (req.other_keys & service_id) {
+	      ldout(cct, 10) << " adding key for service "
+			     << ceph_entity_type_name(service_id) << dendl;
+	      CephXSessionAuthInfo svc_info;
+	      key_server->build_session_auth_info(
+		service_id,
+		info.ticket,
+		svc_info);
+	      svc_info.validity += cct->_conf->auth_service_ticket_ttl;
+	      info_vec.push_back(svc_info);
+	    }
+	  }
+	  bufferlist extra;
+	  if (!info_vec.empty()) {
+	    CryptoKey no_key;
+	    cephx_build_service_ticket_reply(
+	      cct, session_key, info_vec, false, no_key, extra);
+	  }
+	  encode(extra, *result_bl);
+	}
+
+	// caller should try to finish authentication
+	ret = 1;
       }
     }
     break;
@@ -154,8 +225,10 @@ int CephxServiceHandler::handle_request(bufferlist::iterator& indata, bufferlist
       bufferlist tmp_bl;
       CephXServiceTicketInfo auth_ticket_info;
       // note: no challenge here.
-      if (!cephx_verify_authorizer(cct, key_server, indata, auth_ticket_info, nullptr,
-				   tmp_bl)) {
+      if (!cephx_verify_authorizer(
+	    cct, key_server, indata, 0, auth_ticket_info, nullptr,
+	    nullptr,
+	    &tmp_bl)) {
         ret = -EPERM;
 	break;
       }
@@ -174,8 +247,10 @@ int CephxServiceHandler::handle_request(bufferlist::iterator& indata, bufferlist
 	  ldout(cct, 10) << " adding key for service "
 			 << ceph_entity_type_name(service_id) << dendl;
           CephXSessionAuthInfo info;
-          int r = key_server->build_session_auth_info(service_id,
-						      auth_ticket_info, info);
+          int r = key_server->build_session_auth_info(
+	    service_id,
+	    auth_ticket_info.ticket,  // parent ticket (client's auth ticket)
+	    info);
 	  // tolerate missing MGR rotating key for the purposes of upgrades.
           if (r < 0) {
 	    ldout(cct, 10) << "   missing key for service "
@@ -193,16 +268,18 @@ int CephxServiceHandler::handle_request(bufferlist::iterator& indata, bufferlist
 	ret = service_err;
       }
       CryptoKey no_key;
-      build_cephx_response_header(cephx_header.request_type, ret, result_bl);
-      cephx_build_service_ticket_reply(cct, auth_ticket_info.session_key, info_vec, false, no_key, result_bl);
+      build_cephx_response_header(cephx_header.request_type, ret, *result_bl);
+      cephx_build_service_ticket_reply(cct, auth_ticket_info.session_key,
+				       info_vec, false, no_key, *result_bl);
     }
     break;
 
   case CEPHX_GET_ROTATING_KEY:
     {
-      ldout(cct, 10) << "handle_request getting rotating secret for " << entity_name << dendl;
-      build_cephx_response_header(cephx_header.request_type, 0, result_bl);
-      if (!key_server->get_rotating_encrypted(entity_name, result_bl)) {
+      ldout(cct, 10) << "handle_request getting rotating secret for "
+		     << entity_name << dendl;
+      build_cephx_response_header(cephx_header.request_type, 0, *result_bl);
+      if (!key_server->get_rotating_encrypted(entity_name, *result_bl)) {
         ret = -EPERM;
         break;
       }
