@@ -22,14 +22,17 @@ class CephContext;
 struct AuthNoneAuthorizer : public AuthAuthorizer {
   AuthNoneAuthorizer() : AuthAuthorizer(CEPH_AUTH_NONE) { }
   bool build_authorizer(const EntityName &ename, uint64_t global_id) {
-    __u8 struct_v = 1;
+    __u8 struct_v = 1; // see AUTH_MODE_* in Auth.h
     encode(struct_v, bl);
     encode(ename, bl);
     encode(global_id, bl);
     return 0;
   }
-  bool verify_reply(bufferlist::iterator& reply) override { return true; }
-  bool add_challenge(CephContext *cct, bufferlist& ch) override { return true; }
+  bool verify_reply(bufferlist::const_iterator& reply,
+		    std::string *connection_secret) override { return true; }
+  bool add_challenge(CephContext *cct, const bufferlist& ch) override {
+    return true;
+  }
 };
 
 #endif
