@@ -3,10 +3,6 @@
 //  Software License, Version 1.0. (See accompanying file
 //  LICENSE_1_0.txt or copy at http://www.boost.org/LICENSE_1_
 
-#ifdef _MSC_VER
-#  define _SCL_SECURE_NO_WARNINGS
-#endif
-
 #include <boost/multiprecision/cpp_int.hpp>
 
 #include "test_arithmetic.hpp"
