@@ -9,28 +9,30 @@
 // wrappers around scrub types to offer the necessary bits other than
 // the minimal set that the lirados requires
 struct object_id_wrapper : public librados::object_id_t {
-  object_id_wrapper(const hobject_t& hoid)
+  explicit object_id_wrapper(const hobject_t& hoid)
     : object_id_t{hoid.oid.name, hoid.nspace, hoid.get_key(), hoid.snap}
   {}
   void encode(bufferlist& bl) const;
-  void decode(bufferlist::iterator& bl);
+  void decode(bufferlist::const_iterator& bl);
 };
 
 WRITE_CLASS_ENCODER(object_id_wrapper)
 
-inline void decode(librados::object_id_t& obj, bufferlist::iterator& bp) {
+namespace librados {
+inline void decode(object_id_t& obj, bufferlist::const_iterator& bp) {
   reinterpret_cast<object_id_wrapper&>(obj).decode(bp);
 }
+}
 
 struct osd_shard_wrapper : public librados::osd_shard_t {
   void encode(bufferlist& bl) const;
-  void decode(bufferlist::iterator& bp);
+  void decode(bufferlist::const_iterator& bp);
 };
 
 WRITE_CLASS_ENCODER(osd_shard_wrapper)
 
 namespace librados {
-  inline void decode(librados::osd_shard_t& shard, bufferlist::iterator& bp) {
+  inline void decode(librados::osd_shard_t& shard, bufferlist::const_iterator& bp) {
     reinterpret_cast<osd_shard_wrapper&>(shard).decode(bp);
   }
 }
@@ -38,7 +40,7 @@ namespace librados {
 struct shard_info_wrapper : public librados::shard_info_t {
 public:
   shard_info_wrapper() = default;
-  shard_info_wrapper(const ScrubMap::object& object) {
+  explicit shard_info_wrapper(const ScrubMap::object& object) {
     set_object(object);
   }
   void set_object(const ScrubMap::object& object);
@@ -94,20 +96,20 @@ public:
     errors &= ~err_t::DATA_DIGEST_MISMATCH_INFO;
   }
   void encode(bufferlist& bl) const;
-  void decode(bufferlist::iterator& bp);
+  void decode(bufferlist::const_iterator& bp);
 };
 
 WRITE_CLASS_ENCODER(shard_info_wrapper)
 
 namespace librados {
   inline void decode(librados::shard_info_t& shard,
-		     bufferlist::iterator& bp) {
+		     bufferlist::const_iterator& bp) {
     reinterpret_cast<shard_info_wrapper&>(shard).decode(bp);
   }
 }
 
 struct inconsistent_obj_wrapper : librados::inconsistent_obj_t {
-  inconsistent_obj_wrapper(const hobject_t& hoid);
+  explicit inconsistent_obj_wrapper(const hobject_t& hoid);
 
   void set_object_info_inconsistency() {
     errors |= obj_err_t::OBJECT_INFO_INCONSISTENCY;
@@ -133,9 +135,6 @@ struct inconsistent_obj_wrapper : librados::inconsistent_obj_t {
   void set_hinfo_inconsistency() {
     errors |= obj_err_t::HINFO_INCONSISTENCY;
   }
-  void set_size_too_large() {
-    errors |= obj_err_t::SIZE_TOO_LARGE;
-  }
   void add_shard(const pg_shard_t& pgs, const shard_info_wrapper& shard);
   void set_auth_missing(const hobject_t& hoid,
                         const map<pg_shard_t, ScrubMap*>&,
@@ -144,19 +143,19 @@ struct inconsistent_obj_wrapper : librados::inconsistent_obj_t {
 			const pg_shard_t &primary);
   void set_version(uint64_t ver) { version = ver; }
   void encode(bufferlist& bl) const;
-  void decode(bufferlist::iterator& bp);
+  void decode(bufferlist::const_iterator& bp);
 };
 
 WRITE_CLASS_ENCODER(inconsistent_obj_wrapper)
 
 inline void decode(librados::inconsistent_obj_t& obj,
-		   bufferlist::iterator& bp) {
+		   bufferlist::const_iterator& bp) {
   reinterpret_cast<inconsistent_obj_wrapper&>(obj).decode(bp);
 }
 
 struct inconsistent_snapset_wrapper : public librados::inconsistent_snapset_t {
   inconsistent_snapset_wrapper() = default;
-  inconsistent_snapset_wrapper(const hobject_t& head);
+  explicit inconsistent_snapset_wrapper(const hobject_t& head);
   void set_headless();
   // soid claims that it is a head or a snapdir, but its SS_ATTR
   // is missing.
@@ -170,19 +169,17 @@ struct inconsistent_snapset_wrapper : public librados::inconsistent_snapset_t {
   void set_clone(snapid_t);
   // the snapset is not consistent with itself
   void set_snapset_error();
-  // soid.snap inconsistent with snapset
-  void set_head_mismatch();
   void set_size_mismatch();
 
   void encode(bufferlist& bl) const;
-  void decode(bufferlist::iterator& bp);
+  void decode(bufferlist::const_iterator& bp);
 };
 
 WRITE_CLASS_ENCODER(inconsistent_snapset_wrapper)
 
 namespace librados {
   inline void decode(librados::inconsistent_snapset_t& snapset,
-		     bufferlist::iterator& bp) {
+		     bufferlist::const_iterator& bp) {
     reinterpret_cast<inconsistent_snapset_wrapper&>(snapset).decode(bp);
   }
 }
@@ -193,7 +190,7 @@ struct scrub_ls_arg_t {
   librados::object_id_t start_after;
   uint64_t max_return;
   void encode(bufferlist& bl) const;
-  void decode(bufferlist::iterator& bl);
+  void decode(bufferlist::const_iterator& bl);
 };
 
 WRITE_CLASS_ENCODER(scrub_ls_arg_t);
@@ -202,7 +199,7 @@ struct scrub_ls_result_t {
   epoch_t interval;
   std::vector<bufferlist> vals;
   void encode(bufferlist& bl) const;
-  void decode(bufferlist::iterator& bl);
+  void decode(bufferlist::const_iterator& bl);
 };
 
 WRITE_CLASS_ENCODER(scrub_ls_result_t);
