@@ -29,7 +29,7 @@
 
 #include "common/config.h"
 #include "common/errno.h"
-#include "include/assert.h"
+#include "include/ceph_assert.h"
 
 #define dout_context g_ceph_context
 #define dout_subsys ceph_subsys_mds
@@ -96,7 +96,7 @@ class C_MDL_WriteError : public MDSIOContextBase {
   void finish(int r) override {
     MDSRank *mds = get_mds();
     // assume journal is reliable, so don't choose action based on
-    // g_conf->mds_action_on_write_error.
+    // g_conf()->mds_action_on_write_error.
     if (r == -EBLACKLISTED) {
       derr << "we have been blacklisted (fenced), respawning..." << dendl;
       mds->respawn();
@@ -121,7 +121,7 @@ class C_MDL_WriteError : public MDSIOContextBase {
 };
 
 
-void MDLog::write_head(MDSInternalContextBase *c) 
+void MDLog::write_head(MDSContext *c) 
 {
   Context *fin = NULL;
   if (c != NULL) {
@@ -147,7 +147,7 @@ uint64_t MDLog::get_safe_pos() const
 
 
 
-void MDLog::create(MDSInternalContextBase *c)
+void MDLog::create(MDSContext *c)
 {
   dout(5) << "create empty log" << dendl;
 
@@ -160,14 +160,14 @@ void MDLog::create(MDSInternalContextBase *c)
   ino = MDS_INO_LOG_OFFSET + mds->get_nodeid();
 
   // Instantiate Journaler and start async write to RADOS
-  assert(journaler == NULL);
+  ceph_assert(journaler == NULL);
   journaler = new Journaler("mdlog", ino, mds->mdsmap->get_metadata_pool(),
                             CEPH_FS_ONDISK_MAGIC, mds->objecter, logger,
                             l_mdl_jlat, mds->finisher);
-  assert(journaler->is_readonly());
+  ceph_assert(journaler->is_readonly());
   journaler->set_write_error_handler(new C_MDL_WriteError(this));
   journaler->set_writeable();
-  journaler->create(&mds->mdcache->default_log_layout, g_conf->mds_journal_format);
+  journaler->create(&mds->mdcache->default_log_layout, g_conf()->mds_journal_format);
   journaler->write_head(gather.new_sub());
 
   // Async write JournalPointer to RADOS
@@ -184,11 +184,11 @@ void MDLog::create(MDSInternalContextBase *c)
   submit_thread.create("md_submit");
 }
 
-void MDLog::open(MDSInternalContextBase *c)
+void MDLog::open(MDSContext *c)
 {
   dout(5) << "open discovering log bounds" << dendl;
 
-  assert(!recovery_thread.is_started());
+  ceph_assert(!recovery_thread.is_started());
   recovery_thread.set_completion(c);
   recovery_thread.create("md_recov_open");
 
@@ -202,9 +202,9 @@ void MDLog::open(MDSInternalContextBase *c)
  */
 class C_ReopenComplete : public MDSInternalContext {
   MDLog *mdlog;
-  MDSInternalContextBase *on_complete;
+  MDSContext *on_complete;
 public:
-  C_ReopenComplete(MDLog *mdlog_, MDSInternalContextBase *on_complete_) : MDSInternalContext(mdlog_->mds), mdlog(mdlog_), on_complete(on_complete_) {}
+  C_ReopenComplete(MDLog *mdlog_, MDSContext *on_complete_) : MDSInternalContext(mdlog_->mds), mdlog(mdlog_), on_complete(on_complete_) {}
   void finish(int r) override {
     mdlog->append();
     on_complete->complete(r);
@@ -216,14 +216,14 @@ public:
  * recovery procedure again, potentially reformatting the journal if it
  * was in an old format.
  */
-void MDLog::reopen(MDSInternalContextBase *c)
+void MDLog::reopen(MDSContext *c)
 {
   dout(5) << "reopen" << dendl;
 
   // Because we will call append() at the completion of this, check that we have already
   // read the whole journal.
-  assert(journaler != NULL);
-  assert(journaler->get_read_pos() == journaler->get_write_pos());
+  ceph_assert(journaler != NULL);
+  ceph_assert(journaler->get_read_pos() == journaler->get_write_pos());
 
   delete journaler;
   journaler = NULL;
@@ -254,9 +254,9 @@ void MDLog::append()
 
 void MDLog::_start_entry(LogEvent *e)
 {
-  assert(submit_mutex.is_locked_by_me());
+  ceph_assert(submit_mutex.is_locked_by_me());
 
-  assert(cur_event == NULL);
+  ceph_assert(cur_event == NULL);
   cur_event = e;
 
   event_seq++;
@@ -270,22 +270,22 @@ void MDLog::_start_entry(LogEvent *e)
 
 void MDLog::cancel_entry(LogEvent *le)
 {
-  assert(le == cur_event);
+  ceph_assert(le == cur_event);
   cur_event = NULL;
   delete le;
 }
 
 void MDLog::_submit_entry(LogEvent *le, MDSLogContextBase *c)
 {
-  assert(submit_mutex.is_locked_by_me());
-  assert(!mds->is_any_replay());
-  assert(!capped);
+  ceph_assert(submit_mutex.is_locked_by_me());
+  ceph_assert(!mds->is_any_replay());
+  ceph_assert(!capped);
 
-  assert(le == cur_event);
+  ceph_assert(le == cur_event);
   cur_event = NULL;
 
   // let the event register itself in the segment
-  assert(!segments.empty());
+  ceph_assert(!segments.empty());
   LogSegment *ls = segments.rbegin()->second;
   ls->num_events++;
 
@@ -313,11 +313,11 @@ void MDLog::_submit_entry(LogEvent *le, MDSLogContextBase *c)
     // disambiguate imports. Because the ESubtreeMap reflects the subtree
     // state when all EImportFinish events are replayed.
   } else if (ls->end/period != ls->offset/period ||
-	     ls->num_events >= g_conf->mds_log_events_per_segment) {
+	     ls->num_events >= g_conf()->mds_log_events_per_segment) {
     dout(10) << "submit_entry also starting new segment: last = "
 	     << ls->seq  << "/" << ls->offset << ", event seq = " << event_seq << dendl;
     _start_new_segment();
-  } else if (g_conf->mds_debug_subtrees &&
+  } else if (g_conf()->mds_debug_subtrees &&
 	     le->get_type() != EVENT_SUBTREEMAP_TEST) {
     // debug: journal this every time to catch subtree replay bugs.
     // use a different event id so it doesn't get interpreted as a
@@ -335,7 +335,7 @@ class C_MDL_Flushed : public MDSLogContextBase {
 protected:
   MDLog *mdlog;
   MDSRank *get_mds() override {return mdlog->mds;}
-  MDSInternalContextBase *wrapped;
+  MDSContext *wrapped;
 
   void finish(int r) override {
     if (wrapped)
@@ -343,7 +343,7 @@ protected:
   }
 
 public:
-  C_MDL_Flushed(MDLog *m, MDSInternalContextBase *w)
+  C_MDL_Flushed(MDLog *m, MDSContext *w)
     : mdlog(m), wrapped(w) {}
   C_MDL_Flushed(MDLog *m, uint64_t wp) : mdlog(m), wrapped(NULL) {
     set_write_pos(wp);
@@ -357,7 +357,7 @@ void MDLog::_submit_thread()
   submit_mutex.Lock();
 
   while (!mds->is_daemon_stopping()) {
-    if (g_conf->mds_log_pause) {
+    if (g_conf()->mds_log_pause) {
       submit_cond.Wait(submit_mutex);
       continue;
     }
@@ -402,7 +402,7 @@ void MDLog::_submit_thread()
       MDSLogContextBase *fin;
       if (data.fin) {
 	fin = dynamic_cast<MDSLogContextBase*>(data.fin);
-	assert(fin);
+	ceph_assert(fin);
 	fin->set_write_pos(new_write_pos);
       } else {
 	fin = new C_MDL_Flushed(this, new_write_pos);
@@ -419,9 +419,9 @@ void MDLog::_submit_thread()
       delete le;
     } else {
       if (data.fin) {
-	MDSInternalContextBase* fin =
-		dynamic_cast<MDSInternalContextBase*>(data.fin);
-	assert(fin);
+	MDSContext* fin =
+		dynamic_cast<MDSContext*>(data.fin);
+	ceph_assert(fin);
 	C_MDL_Flushed *fin2 = new C_MDL_Flushed(this, fin);
 	fin2->set_write_pos(journaler->get_write_pos());
 	journaler->wait_for_flush(fin2);
@@ -440,7 +440,7 @@ void MDLog::_submit_thread()
   submit_mutex.Unlock();
 }
 
-void MDLog::wait_for_safe(MDSInternalContextBase *c)
+void MDLog::wait_for_safe(MDSContext *c)
 {
   submit_mutex.Lock();
 
@@ -477,7 +477,7 @@ void MDLog::flush()
 
 void MDLog::kick_submitter()
 {
-  Mutex::Locker l(submit_mutex);
+  std::lock_guard l(submit_mutex);
   submit_cond.Signal();
 }
 
@@ -489,11 +489,11 @@ void MDLog::cap()
 
 void MDLog::shutdown()
 {
-  assert(mds->mds_lock.is_locked_by_me());
+  ceph_assert(mds->mds_lock.is_locked_by_me());
 
   dout(5) << "shutdown" << dendl;
   if (submit_thread.is_started()) {
-    assert(mds->is_daemon_stopping());
+    ceph_assert(mds->is_daemon_stopping());
 
     if (submit_thread.am_self()) {
       // Called suicide from the thread: trust it to do no work after
@@ -545,7 +545,7 @@ void MDLog::_start_new_segment()
 
 void MDLog::_prepare_new_segment()
 {
-  assert(submit_mutex.is_locked_by_me());
+  ceph_assert(submit_mutex.is_locked_by_me());
 
   uint64_t seq = event_seq + 1;
   dout(7) << __func__ << " seq " << seq << dendl;
@@ -561,9 +561,9 @@ void MDLog::_prepare_new_segment()
   mds->mdcache->advance_stray();
 }
 
-void MDLog::_journal_segment_subtree_map(MDSInternalContextBase *onsync)
+void MDLog::_journal_segment_subtree_map(MDSContext *onsync)
 {
-  assert(submit_mutex.is_locked_by_me());
+  ceph_assert(submit_mutex.is_locked_by_me());
 
   dout(7) << __func__ << dendl;
   ESubtreeMap *sle = mds->mdcache->create_subtree_map();
@@ -572,10 +572,21 @@ void MDLog::_journal_segment_subtree_map(MDSInternalContextBase *onsync)
   _submit_entry(sle, new C_MDL_Flushed(this, onsync));
 }
 
+class C_OFT_Committed : public MDSInternalContext {
+  MDLog *mdlog;
+  uint64_t seq;
+public:
+  C_OFT_Committed(MDLog *l, uint64_t s) :
+    MDSInternalContext(l->mds), mdlog(l), seq(s) {}
+  void finish(int ret) override {
+    mdlog->trim_expired_segments();
+  }
+};
+
 void MDLog::trim(int m)
 {
-  unsigned max_segments = g_conf->mds_log_max_segments;
-  int max_events = g_conf->mds_log_max_events;
+  unsigned max_segments = g_conf()->mds_log_max_segments;
+  int max_events = g_conf()->mds_log_max_events;
   if (m >= 0)
     max_events = m;
 
@@ -585,8 +596,8 @@ void MDLog::trim(int m)
   }
 
   // Clamp max_events to not be smaller than events per segment
-  if (max_events > 0 && max_events <= g_conf->mds_log_events_per_segment) {
-    max_events = g_conf->mds_log_events_per_segment + 1;
+  if (max_events > 0 && max_events <= g_conf()->mds_log_events_per_segment) {
+    max_events = g_conf()->mds_log_events_per_segment + 1;
   }
 
   submit_mutex.Lock();
@@ -644,7 +655,7 @@ void MDLog::trim(int m)
     
     // look at first segment
     LogSegment *ls = p->second;
-    assert(ls);
+    ceph_assert(ls);
     ++p;
     
     if (pending_events.count(ls->seq) ||
@@ -653,6 +664,7 @@ void MDLog::trim(int m)
 	      << journaler->get_write_safe_pos() << " < end " << ls->end << dendl;
       break;
     }
+
     if (expiring_segments.count(ls)) {
       dout(5) << "trim already expiring segment " << ls->seq << "/" << ls->offset
 	      << ", " << ls->num_events << " events" << dendl;
@@ -660,7 +672,7 @@ void MDLog::trim(int m)
       dout(5) << "trim already expired segment " << ls->seq << "/" << ls->offset
 	      << ", " << ls->num_events << " events" << dendl;
     } else {
-      assert(expiring_segments.count(ls) == 0);
+      ceph_assert(expiring_segments.count(ls) == 0);
       new_expiring_segments++;
       expiring_segments.insert(ls);
       expiring_events += ls->num_events;
@@ -674,6 +686,18 @@ void MDLog::trim(int m)
     }
   }
 
+  if (!capped &&
+      !mds->mdcache->open_file_table.is_any_committing()) {
+    uint64_t last_seq = get_last_segment_seq();
+    if (mds->mdcache->open_file_table.is_any_dirty() ||
+	last_seq > mds->mdcache->open_file_table.get_committed_log_seq()) {
+      submit_mutex.Unlock();
+      mds->mdcache->open_file_table.commit(new C_OFT_Committed(this, last_seq),
+					   last_seq, CEPH_MSG_PRIO_HIGH);
+      submit_mutex.Lock();
+    }
+  }
+
   // discard expired segments and unlock submit_mutex
   _trim_expired_segments();
 }
@@ -706,8 +730,17 @@ int MDLog::trim_all()
            << "/" << expired_segments.size() << dendl;
 
   uint64_t last_seq = 0;
-  if (!segments.empty())
+  if (!segments.empty()) {
     last_seq = get_last_segment_seq();
+    if (!capped &&
+	!mds->mdcache->open_file_table.is_any_committing() &&
+	last_seq > mds->mdcache->open_file_table.get_committing_log_seq()) {
+      submit_mutex.Unlock();
+      mds->mdcache->open_file_table.commit(new C_OFT_Committed(this, last_seq),
+					   last_seq, CEPH_MSG_PRIO_DEFAULT);
+      submit_mutex.Lock();
+    }
+  }
 
   map<uint64_t,LogSegment*>::iterator p = segments.begin();
   while (p != segments.end() &&
@@ -730,7 +763,7 @@ int MDLog::trim_all()
       dout(5) << "trim already expired segment " << ls->seq << "/" << ls->offset
 	      << ", " << ls->num_events << " events" << dendl;
     } else {
-      assert(expiring_segments.count(ls) == 0);
+      ceph_assert(expiring_segments.count(ls) == 0);
       expiring_segments.insert(ls);
       expiring_events += ls->num_events;
       submit_mutex.Unlock();
@@ -761,7 +794,7 @@ void MDLog::try_expire(LogSegment *ls, int op_prio)
   } else {
     dout(10) << "try_expire expired segment " << ls->seq << "/" << ls->offset << dendl;
     submit_mutex.Lock();
-    assert(expiring_segments.count(ls));
+    ceph_assert(expiring_segments.count(ls));
     expiring_segments.erase(ls);
     expiring_events -= ls->num_events;
     _expired(ls);
@@ -786,7 +819,9 @@ void MDLog::_maybe_expired(LogSegment *ls, int op_prio)
 
 void MDLog::_trim_expired_segments()
 {
-  assert(submit_mutex.is_locked_by_me());
+  ceph_assert(submit_mutex.is_locked_by_me());
+
+  uint64_t oft_committed_seq = mds->mdcache->open_file_table.get_committed_log_seq();
 
   // trim expired segments?
   bool trimmed = false;
@@ -797,6 +832,12 @@ void MDLog::_trim_expired_segments()
 	       << " to expire" << dendl;
       break;
     }
+
+    if (!capped && ls->seq >= oft_committed_seq) {
+      dout(10) << "_trim_expired_segments open file table committedseq " << oft_committed_seq
+	       << " <= " << ls->seq << "/" << ls->offset << dendl;
+      break;
+    }
     
     dout(10) << "_trim_expired_segments trimming expired "
 	     << ls->seq << "/0x" << std::hex << ls->offset << std::dec << dendl;
@@ -836,7 +877,7 @@ void MDLog::trim_expired_segments()
 
 void MDLog::_expired(LogSegment *ls)
 {
-  assert(submit_mutex.is_locked_by_me());
+  ceph_assert(submit_mutex.is_locked_by_me());
 
   dout(5) << "_expired segment " << ls->seq << "/" << ls->offset
 	  << ", " << ls->num_events << " events" << dendl;
@@ -850,11 +891,7 @@ void MDLog::_expired(LogSegment *ls)
     expired_events += ls->num_events;
 
     // Trigger all waiters
-    for (std::list<MDSInternalContextBase*>::iterator i = ls->expiry_waiters.begin();
-        i != ls->expiry_waiters.end(); ++i) {
-      (*i)->complete(0);
-    }
-    ls->expiry_waiters.clear();
+    finish_contexts(g_ceph_context, ls->expiry_waiters);
     
     logger->inc(l_mdl_evex, ls->num_events);
     logger->inc(l_mdl_segex);
@@ -868,10 +905,10 @@ void MDLog::_expired(LogSegment *ls)
 
 
 
-void MDLog::replay(MDSInternalContextBase *c)
+void MDLog::replay(MDSContext *c)
 {
-  assert(journaler->is_active());
-  assert(journaler->is_readonly());
+  ceph_assert(journaler->is_active());
+  ceph_assert(journaler->is_readonly());
 
   // empty?
   if (journaler->get_read_pos() == journaler->get_write_pos()) {
@@ -893,7 +930,7 @@ void MDLog::replay(MDSInternalContextBase *c)
   dout(10) << "replay start, from " << journaler->get_read_pos()
 	   << " to " << journaler->get_write_pos() << dendl;
 
-  assert(num_events == 0 || already_replayed);
+  ceph_assert(num_events == 0 || already_replayed);
   if (already_replayed) {
     // Ensure previous instance of ReplayThread is joined before
     // we create another one
@@ -917,10 +954,10 @@ void MDLog::replay(MDSInternalContextBase *c)
  * When this function completes, the `journaler` attribute will be set to
  * a Journaler instance using the latest available serialization format.
  */
-void MDLog::_recovery_thread(MDSInternalContextBase *completion)
+void MDLog::_recovery_thread(MDSContext *completion)
 {
-  assert(journaler == NULL);
-  if (g_conf->mds_journal_format > JOURNAL_FORMAT_MAX) {
+  ceph_assert(journaler == NULL);
+  if (g_conf()->mds_journal_format > JOURNAL_FORMAT_MAX) {
       dout(0) << "Configuration value for mds_journal_format is out of bounds, max is "
               << JOURNAL_FORMAT_MAX << dendl;
 
@@ -940,7 +977,7 @@ void MDLog::_recovery_thread(MDSInternalContextBase *completion)
     jp.front = default_log_ino;
     int write_result = jp.save(mds->objecter);
     // Nothing graceful we can do for this
-    assert(write_result >= 0);
+    ceph_assert(write_result >= 0);
   } else if (read_result == -EBLACKLISTED) {
     derr << "Blacklisted during JournalPointer read!  Respawning..." << dendl;
     mds->respawn();
@@ -959,7 +996,7 @@ void MDLog::_recovery_thread(MDSInternalContextBase *completion)
     if (mds->is_standby_replay()) {
       dout(1) << "Journal " << jp.front << " is being rewritten, "
         << "cannot replay in standby until an active MDS completes rewrite" << dendl;
-      Mutex::Locker l(mds->mds_lock);
+      std::lock_guard l(mds->mds_lock);
       if (mds->is_daemon_stopping()) {
         return;
       }
@@ -986,7 +1023,7 @@ void MDLog::_recovery_thread(MDSInternalContextBase *completion)
       mds->clog->error() << "Error recovering journal " << jp.front << ": "
         << cpp_strerror(recovery_result);
       mds->damaged_unlocked();
-      assert(recovery_result == 0); // Unreachable because damaged() calls respawn()
+      ceph_assert(recovery_result == 0); // Unreachable because damaged() calls respawn()
     }
 
     // We could read journal, so we can erase it.
@@ -1002,7 +1039,7 @@ void MDLog::_recovery_thread(MDSInternalContextBase *completion)
       jp.back = 0;
       int write_result = jp.save(mds->objecter);
       // Nothing graceful we can do for this
-      assert(write_result >= 0);
+      ceph_assert(write_result >= 0);
     }
   }
 
@@ -1014,7 +1051,7 @@ void MDLog::_recovery_thread(MDSInternalContextBase *completion)
   // Assign to ::journaler so that we can be aborted by ::shutdown while
   // waiting for journaler recovery
   {
-    Mutex::Locker l(mds->mds_lock);
+    std::lock_guard l(mds->mds_lock);
     journaler = front_journal;
   }
 
@@ -1032,7 +1069,7 @@ void MDLog::_recovery_thread(MDSInternalContextBase *completion)
     mds->clog->error() << "Error recovering journal " << jp.front << ": "
       << cpp_strerror(recovery_result);
     mds->damaged_unlocked();
-    assert(recovery_result == 0); // Unreachable because damaged() calls respawn()
+    ceph_assert(recovery_result == 0); // Unreachable because damaged() calls respawn()
   }
 
   /* Check whether the front journal format is acceptable or needs re-write */
@@ -1040,7 +1077,7 @@ void MDLog::_recovery_thread(MDSInternalContextBase *completion)
     dout(0) << "Journal " << jp.front << " is in unknown format " << front_journal->get_stream_format()
             << ", does this MDS daemon require upgrade?" << dendl;
     {
-      Mutex::Locker l(mds->mds_lock);
+      std::lock_guard l(mds->mds_lock);
       if (mds->is_daemon_stopping()) {
         journaler = NULL;
         delete front_journal;
@@ -1048,14 +1085,14 @@ void MDLog::_recovery_thread(MDSInternalContextBase *completion)
       }
       completion->complete(-EINVAL);
     }
-  } else if (mds->is_standby_replay() || front_journal->get_stream_format() >= g_conf->mds_journal_format) {
+  } else if (mds->is_standby_replay() || front_journal->get_stream_format() >= g_conf()->mds_journal_format) {
     /* The journal is of configured format, or we are in standbyreplay and will
      * tolerate replaying old journals until we have to go active. Use front_journal as
      * our journaler attribute and complete */
     dout(4) << "Recovered journal " << jp.front << " in format " << front_journal->get_stream_format() << dendl;
     journaler->set_write_error_handler(new C_MDL_WriteError(this));
     {
-      Mutex::Locker l(mds->mds_lock);
+      std::lock_guard l(mds->mds_lock);
       if (mds->is_daemon_stopping()) {
         return;
       }
@@ -1078,11 +1115,11 @@ void MDLog::_recovery_thread(MDSInternalContextBase *completion)
  * swapping pointers to make that one the front journal only when we have
  * safely completed.
  */
-void MDLog::_reformat_journal(JournalPointer const &jp_in, Journaler *old_journal, MDSInternalContextBase *completion)
+void MDLog::_reformat_journal(JournalPointer const &jp_in, Journaler *old_journal, MDSContext *completion)
 {
-  assert(!jp_in.is_null());
-  assert(completion != NULL);
-  assert(old_journal != NULL);
+  ceph_assert(!jp_in.is_null());
+  ceph_assert(completion != NULL);
+  ceph_assert(old_journal != NULL);
 
   JournalPointer jp = jp_in;
 
@@ -1091,7 +1128,7 @@ void MDLog::_reformat_journal(JournalPointer const &jp_in, Journaler *old_journa
   inodeno_t secondary_ino = MDS_INO_LOG_BACKUP_OFFSET + mds->get_nodeid();
   jp.back = (jp.front == primary_ino ? secondary_ino : primary_ino);
   int write_result = jp.save(mds->objecter);
-  assert(write_result == 0);
+  ceph_assert(write_result == 0);
 
   /* Create the new Journaler file */
   Journaler *new_journal = new Journaler("mdlog", jp.back,
@@ -1099,7 +1136,7 @@ void MDLog::_reformat_journal(JournalPointer const &jp_in, Journaler *old_journa
   dout(4) << "Writing new journal header " << jp.back << dendl;
   file_layout_t new_layout = old_journal->get_layout();
   new_journal->set_writeable();
-  new_journal->create(&new_layout, g_conf->mds_journal_format);
+  new_journal->create(&new_layout, g_conf()->mds_journal_format);
 
   /* Write the new journal header to RADOS */
   C_SaferCond write_head_wait;
@@ -1144,22 +1181,22 @@ void MDLog::_reformat_journal(JournalPointer const &jp_in, Journaler *old_journa
       break;
 
     // Read one serialized LogEvent
-    assert(old_journal->is_readable());
+    ceph_assert(old_journal->is_readable());
     bufferlist bl;
     uint64_t le_pos = old_journal->get_read_pos();
     bool r = old_journal->try_read_entry(bl);
     if (!r && old_journal->get_error())
       continue;
-    assert(r);
+    ceph_assert(r);
 
     // Update segment_pos_rewrite
-    LogEvent *le = LogEvent::decode(bl);
+    auto le = LogEvent::decode_event(bl.cbegin());
     if (le) {
       bool modified = false;
 
       if (le->get_type() == EVENT_SUBTREEMAP ||
           le->get_type() == EVENT_RESETJOURNAL) {
-        ESubtreeMap *sle = dynamic_cast<ESubtreeMap*>(le);
+        auto sle = dynamic_cast<ESubtreeMap*>(le.get());
         if (sle == NULL || sle->event_seq == 0) {
           // A non-explicit event seq: the effective sequence number 
           // of this segment is it's position in the old journal and
@@ -1183,11 +1220,10 @@ void MDLog::_reformat_journal(JournalPointer const &jp_in, Journaler *old_journa
       // (expire_pos is just an optimization so it's safe to eliminate it)
       if (le->get_type() == EVENT_SUBTREEMAP
           || le->get_type() == EVENT_SUBTREEMAP_TEST) {
-        ESubtreeMap *sle = dynamic_cast<ESubtreeMap*>(le);
-        assert(sle != NULL);
+        auto& sle = dynamic_cast<ESubtreeMap&>(*le);
         dout(20) << __func__ << " zeroing expire_pos in subtreemap event at "
-          << le_pos << " seq=" << sle->event_seq << dendl;
-        sle->expire_pos = 0;
+          << le_pos << " seq=" << sle.event_seq << dendl;
+        sle.expire_pos = 0;
         modified = true;
       }
 
@@ -1195,8 +1231,6 @@ void MDLog::_reformat_journal(JournalPointer const &jp_in, Journaler *old_journa
         bl.clear();
         le->encode_with_header(bl, mds->mdsmap->get_up_features());
       }
-
-      delete le;
     } else {
       // Failure from LogEvent::decode, our job is to change the journal wrapper,
       // not validate the contents, so pass it through.
@@ -1217,28 +1251,28 @@ void MDLog::_reformat_journal(JournalPointer const &jp_in, Journaler *old_journa
 
   // If failed to rewrite journal, leave the part written journal
   // as garbage to be cleaned up next startup.
-  assert(r == 0);
+  ceph_assert(r == 0);
 
   /* Now that the new journal is safe, we can flip the pointers */
   inodeno_t const tmp = jp.front;
   jp.front = jp.back;
   jp.back = tmp;
   write_result = jp.save(mds->objecter);
-  assert(write_result == 0);
+  ceph_assert(write_result == 0);
 
   /* Delete the old journal to free space */
   dout(1) << "New journal flushed, erasing old journal" << dendl;
   C_SaferCond erase_waiter;
   old_journal->erase(&erase_waiter);
   int erase_result = erase_waiter.wait();
-  assert(erase_result == 0);
+  ceph_assert(erase_result == 0);
   {
-    Mutex::Locker l(mds->mds_lock);
+    std::lock_guard l(mds->mds_lock);
     if (mds->is_daemon_stopping()) {
       delete new_journal;
       return;
     }
-    assert(journaler == old_journal);
+    ceph_assert(journaler == old_journal);
     journaler = NULL;
     delete old_journal;
   }
@@ -1246,12 +1280,12 @@ void MDLog::_reformat_journal(JournalPointer const &jp_in, Journaler *old_journa
   /* Update the pointer to reflect we're back in clean single journal state. */
   jp.back = 0;
   write_result = jp.save(mds->objecter);
-  assert(write_result == 0);
+  ceph_assert(write_result == 0);
 
   /* Reset the Journaler object to its default state */
   dout(1) << "Journal rewrite complete, continuing with normal startup" << dendl;
   {
-    Mutex::Locker l(mds->mds_lock);
+    std::lock_guard l(mds->mds_lock);
     if (mds->is_daemon_stopping()) {
       delete new_journal;
       return;
@@ -1263,7 +1297,7 @@ void MDLog::_reformat_journal(JournalPointer const &jp_in, Journaler *old_journa
 
   /* Trigger completion */
   {
-    Mutex::Locker l(mds->mds_lock);
+    std::lock_guard l(mds->mds_lock);
     if (mds->is_daemon_stopping()) {
       return;
     }
@@ -1350,7 +1384,7 @@ void MDLog::_replay_thread()
 	journaler->get_read_pos() == journaler->get_write_pos())
       break;
     
-    assert(journaler->is_readable() || mds->is_daemon_stopping());
+    ceph_assert(journaler->is_readable() || mds->is_daemon_stopping());
     
     // read it
     uint64_t pos = journaler->get_read_pos();
@@ -1358,10 +1392,10 @@ void MDLog::_replay_thread()
     bool r = journaler->try_read_entry(bl);
     if (!r && journaler->get_error())
       continue;
-    assert(r);
+    ceph_assert(r);
     
     // unpack event
-    LogEvent *le = LogEvent::decode(bl);
+    auto le = LogEvent::decode_event(bl.cbegin());
     if (!le) {
       dout(0) << "_replay " << pos << "~" << bl.length() << " / " << journaler->get_write_pos() 
 	      << " -- unable to decode event" << dendl;
@@ -1372,7 +1406,7 @@ void MDLog::_replay_thread()
       mds->clog->error() << "corrupt journal event at " << pos << "~"
                          << bl.length() << " / "
                          << journaler->get_write_pos();
-      if (g_conf->mds_log_skip_corrupt_events) {
+      if (g_conf()->mds_log_skip_corrupt_events) {
         continue;
       } else {
         mds->damaged_unlocked();
@@ -1386,7 +1420,7 @@ void MDLog::_replay_thread()
     // new segment?
     if (le->get_type() == EVENT_SUBTREEMAP ||
 	le->get_type() == EVENT_RESETJOURNAL) {
-      ESubtreeMap *sle = dynamic_cast<ESubtreeMap*>(le);
+      auto sle = dynamic_cast<ESubtreeMap*>(le.get());
       if (sle && sle->event_seq > 0)
 	event_seq = sle->event_seq;
       else
@@ -1410,7 +1444,7 @@ void MDLog::_replay_thread()
       num_events++;
 
       {
-        Mutex::Locker l(mds->mds_lock);
+        std::lock_guard l(mds->mds_lock);
         if (mds->is_daemon_stopping()) {
           return;
         }
@@ -1418,14 +1452,13 @@ void MDLog::_replay_thread()
         le->replay(mds);
       }
     }
-    delete le;
 
     logger->set(l_mdl_rdpos, pos);
   }
 
   // done!
   if (r == 0) {
-    assert(journaler->get_read_pos() == journaler->get_write_pos());
+    ceph_assert(journaler->get_read_pos() == journaler->get_write_pos());
     dout(10) << "_replay - complete, " << num_events
 	     << " events" << dendl;
 
@@ -1436,7 +1469,7 @@ void MDLog::_replay_thread()
 
   dout(10) << "_replay_thread kicking waiters" << dendl;
   {
-    Mutex::Locker l(mds->mds_lock);
+    std::lock_guard l(mds->mds_lock);
     if (mds->is_daemon_stopping()) {
       return;
     }
@@ -1452,6 +1485,9 @@ void MDLog::standby_trim_segments()
   dout(10) << "standby_trim_segments" << dendl;
   uint64_t expire_pos = journaler->get_expire_pos();
   dout(10) << " expire_pos=" << expire_pos << dendl;
+
+  mds->mdcache->open_file_table.trim_destroyed_inos(expire_pos);
+
   bool removed_segment = false;
   while (have_any_segments()) {
     LogSegment *seg = get_oldest_segment();
