@@ -37,6 +37,7 @@ CompatSet MDSMap::get_compat_set_all() {
   feature_incompat.insert(MDS_FEATURE_INCOMPAT_INLINE);
   feature_incompat.insert(MDS_FEATURE_INCOMPAT_NOANCHOR);
   feature_incompat.insert(MDS_FEATURE_INCOMPAT_FILE_LAYOUT_V2);
+  feature_incompat.insert(MDS_FEATURE_INCOMPAT_SNAPREALM_V2);
 
   return CompatSet(feature_compat, feature_ro_compat, feature_incompat);
 }
@@ -53,6 +54,7 @@ CompatSet MDSMap::get_compat_set_default() {
   feature_incompat.insert(MDS_FEATURE_INCOMPAT_OMAPDIRFRAG);
   feature_incompat.insert(MDS_FEATURE_INCOMPAT_NOANCHOR);
   feature_incompat.insert(MDS_FEATURE_INCOMPAT_FILE_LAYOUT_V2);
+  feature_incompat.insert(MDS_FEATURE_INCOMPAT_SNAPREALM_V2);
 
   return CompatSet(feature_compat, feature_ro_compat, feature_incompat);
 }
@@ -75,14 +77,11 @@ void MDSMap::mds_info_t::dump(Formatter *f) const
   f->dump_int("incarnation", inc);
   f->dump_stream("state") << ceph_mds_state_name(state);
   f->dump_int("state_seq", state_seq);
-  f->dump_stream("addr") << addr;
+  f->dump_stream("addr") << addrs.get_legacy_str();
+  f->dump_object("addrs", addrs);
   if (laggy_since != utime_t())
     f->dump_stream("laggy_since") << laggy_since;
   
-  f->dump_int("standby_for_rank", standby_for_rank);
-  f->dump_int("standby_for_fscid", standby_for_fscid);
-  f->dump_string("standby_for_name", standby_for_name);
-  f->dump_bool("standby_replay", standby_replay);
   f->open_array_section("export_targets");
   for (set<mds_rank_t>::iterator p = export_targets.begin();
        p != export_targets.end(); ++p) {
@@ -90,12 +89,13 @@ void MDSMap::mds_info_t::dump(Formatter *f) const
   }
   f->close_section();
   f->dump_unsigned("features", mds_features);
+  f->dump_unsigned("flags", flags);
 }
 
 void MDSMap::mds_info_t::print_summary(ostream &out) const
 {
   out << global_id << ":\t"
-      << addr
+      << addrs
       << " '" << name << "'"
       << " mds." << rank
       << "." << inc
@@ -104,19 +104,12 @@ void MDSMap::mds_info_t::print_summary(ostream &out) const
   if (laggy()) {
     out << " laggy since " << laggy_since;
   }
-  if (standby_for_rank != -1 ||
-      !standby_for_name.empty()) {
-    out << " (standby for";
-    //if (standby_for_rank >= 0)
-      out << " rank " << standby_for_rank;
-    if (!standby_for_name.empty()) {
-      out << " '" << standby_for_name << "'";
-    }
-    out << ")";
-  }
   if (!export_targets.empty()) {
     out << " export_targets=" << export_targets;
   }
+  if (is_frozen()) {
+    out << " frozen";
+  }
 }
 
 void MDSMap::mds_info_t::generate_test_instances(list<mds_info_t*>& ls)
@@ -142,6 +135,8 @@ void MDSMap::dump(Formatter *f) const
   f->dump_int("root", root);
   f->dump_int("session_timeout", session_timeout);
   f->dump_int("session_autoclose", session_autoclose);
+  f->dump_stream("min_compat_client") << (int)min_compat_client << " ("
+				      << ceph_release_name(min_compat_client) << ")";
   f->dump_int("max_file_size", max_file_size);
   f->dump_int("last_failure", last_failure);
   f->dump_int("last_failure_osd_epoch", last_failure_osd_epoch);
@@ -220,6 +215,8 @@ void MDSMap::print(ostream& out) const
   out << "session_timeout\t" << session_timeout << "\n"
       << "session_autoclose\t" << session_autoclose << "\n";
   out << "max_file_size\t" << max_file_size << "\n";
+  out << "min_compat_client\t" << (int)min_compat_client << " ("
+			       << ceph_release_name(min_compat_client) << ")\n";
   out << "last_failure\t" << last_failure << "\n"
       << "last_failure_osd_epoch\t" << last_failure_osd_epoch << "\n";
   out << "compat\t" << compat << "\n";
@@ -367,19 +364,36 @@ void MDSMap::get_health(list<pair<health_status_t,string> >& summary,
 	map<mds_gid_t,mds_info_t>::const_iterator info = mds_info.find(gid);
 	stringstream ss;
 	if (is_resolve(i))
-	  ss << "mds." << info->second.name << " at " << info->second.addr << " rank " << i << " is resolving";
+	  ss << "mds." << info->second.name << " at " << info->second.addrs
+	     << " rank " << i << " is resolving";
 	if (is_replay(i))
-	  ss << "mds." << info->second.name << " at " << info->second.addr << " rank " << i << " is replaying journal";
+	  ss << "mds." << info->second.name << " at " << info->second.addrs
+	     << " rank " << i << " is replaying journal";
 	if (is_rejoin(i))
-	  ss << "mds." << info->second.name << " at " << info->second.addr << " rank " << i << " is rejoining";
+	  ss << "mds." << info->second.name << " at " << info->second.addrs
+	     << " rank " << i << " is rejoining";
 	if (is_reconnect(i))
-	  ss << "mds." << info->second.name << " at " << info->second.addr << " rank " << i << " is reconnecting to clients";
+	  ss << "mds." << info->second.name << " at " << info->second.addrs
+	     << " rank " << i << " is reconnecting to clients";
 	if (ss.str().length())
 	  detail->push_back(make_pair(HEALTH_WARN, ss.str()));
       }
     }
   }
 
+  {
+  stringstream ss;
+  ss << fs_name << " max_mds " << max_mds;
+  summary.push_back(make_pair(HEALTH_WARN, ss.str()));
+  }
+
+  if ((mds_rank_t)up.size() < max_mds) {
+    stringstream ss;
+    ss << fs_name << " has " << up.size()
+       << " active MDS(s), but has max_mds of " << max_mds;
+    summary.push_back(make_pair(HEALTH_WARN, ss.str()));
+  }
+
   map<mds_gid_t, mds_info_t>::const_iterator m_end = mds_info.end();
   set<string> laggy;
   for (const auto &u : up) {
@@ -387,13 +401,14 @@ void MDSMap::get_health(list<pair<health_status_t,string> >& summary,
     if (m == m_end) {
       std::cerr << "Up rank " << u.first << " GID " << u.second << " not found!" << std::endl;
     }
-    assert(m != m_end);
+    ceph_assert(m != m_end);
     const mds_info_t &mds_info(m->second);
     if (mds_info.laggy()) {
       laggy.insert(mds_info.name);
       if (detail) {
 	std::ostringstream oss;
-	oss << "mds." << mds_info.name << " at " << mds_info.addr << " is laggy/unresponsive";
+	oss << "mds." << mds_info.name << " at " << mds_info.addrs
+	    << " is laggy/unresponsive";
 	detail->push_back(make_pair(HEALTH_WARN, oss.str()));
       }
     }
@@ -406,6 +421,13 @@ void MDSMap::get_health(list<pair<health_status_t,string> >& summary,
 	<< " laggy";
     summary.push_back(make_pair(HEALTH_WARN, oss.str()));
   }
+
+  if (get_max_mds() > 1 &&
+      was_snaps_ever_allowed() && !allows_multimds_snaps()) {
+    std::ostringstream oss;
+    oss << "multi-active mds while there are snapshots possibly created by pre-mimic MDS";
+    summary.push_back(make_pair(HEALTH_WARN, oss.str()));
+  }
 }
 
 void MDSMap::get_health_checks(health_check_map_t *checks) const
@@ -438,7 +460,7 @@ void MDSMap::get_health_checks(health_check_map_t *checks) const
       map<mds_gid_t,mds_info_t>::const_iterator info = mds_info.find(gid);
       stringstream ss;
       ss << "fs " << fs_name << " mds." << info->second.name << " at "
-	 << info->second.addr << " rank " << i;
+	 << info->second.addrs << " rank " << i;
       if (is_resolve(i))
 	ss << " is resolving";
       if (is_replay(i))
@@ -451,67 +473,121 @@ void MDSMap::get_health_checks(health_check_map_t *checks) const
 	detail.push_back(ss.str());
     }
   }
+
+  // MDS_UP_LESS_THAN_MAX
+  if ((mds_rank_t)get_num_in_mds() < get_max_mds()) {
+    health_check_t& check = checks->add(
+      "MDS_UP_LESS_THAN_MAX", HEALTH_WARN,
+      "%num% filesystem%plurals% %isorare% online with fewer MDS than max_mds");
+    stringstream ss;
+    ss << "fs " << fs_name << " has " << get_num_in_mds()
+       << " MDS online, but wants " << get_max_mds();
+    check.detail.push_back(ss.str());
+  }
+
+  // MDS_ALL_DOWN
+  if ((mds_rank_t)get_num_up_mds() == 0 && get_max_mds() > 0) {
+    health_check_t &check = checks->add(
+      "MDS_ALL_DOWN", HEALTH_ERR,
+      "%num% filesystem%plurals% %isorare% offline");
+    stringstream ss;
+    ss << "fs " << fs_name << " is offline because no MDS is active for it.";
+    check.detail.push_back(ss.str());
+  }
+
+  if (get_max_mds() > 1 &&
+      was_snaps_ever_allowed() && !allows_multimds_snaps()) {
+    health_check_t &check = checks->add(
+      "MULTIMDS_WITH_OLDSNAPS", HEALTH_ERR,
+      "%num% filesystem%plurals% %isorare% multi-active mds with old snapshots");
+    stringstream ss;
+    ss << "multi-active mds while there are snapshots possibly created by pre-mimic MDS";
+    check.detail.push_back(ss.str());
+  }
 }
 
 void MDSMap::mds_info_t::encode_versioned(bufferlist& bl, uint64_t features) const
 {
-  ENCODE_START(7, 4, bl);
-  ::encode(global_id, bl);
-  ::encode(name, bl);
-  ::encode(rank, bl);
-  ::encode(inc, bl);
-  ::encode((int32_t)state, bl);
-  ::encode(state_seq, bl);
-  ::encode(addr, bl, features);
-  ::encode(laggy_since, bl);
-  ::encode(standby_for_rank, bl);
-  ::encode(standby_for_name, bl);
-  ::encode(export_targets, bl);
-  ::encode(mds_features, bl);
-  ::encode(standby_for_fscid, bl);
-  ::encode(standby_replay, bl);
+  __u8 v = 9;
+  if (!HAVE_FEATURE(features, SERVER_NAUTILUS)) {
+    v = 7;
+  }
+  ENCODE_START(v, 4, bl);
+  encode(global_id, bl);
+  encode(name, bl);
+  encode(rank, bl);
+  encode(inc, bl);
+  encode((int32_t)state, bl);
+  encode(state_seq, bl);
+  if (v < 8) {
+    encode(addrs.legacy_addr(), bl, features);
+  } else {
+    encode(addrs, bl, features);
+  }
+  encode(laggy_since, bl);
+  encode(MDS_RANK_NONE, bl); /* standby_for_rank */
+  encode(std::string(), bl); /* standby_for_name */
+  encode(export_targets, bl);
+  encode(mds_features, bl);
+  encode(FS_CLUSTER_ID_NONE, bl); /* standby_for_fscid */
+  encode(false, bl);
+  if (v >= 9) {
+    encode(flags, bl);
+  }
   ENCODE_FINISH(bl);
 }
 
 void MDSMap::mds_info_t::encode_unversioned(bufferlist& bl) const
 {
   __u8 struct_v = 3;
-  ::encode(struct_v, bl);
-  ::encode(global_id, bl);
-  ::encode(name, bl);
-  ::encode(rank, bl);
-  ::encode(inc, bl);
-  ::encode((int32_t)state, bl);
-  ::encode(state_seq, bl);
-  ::encode(addr, bl, 0);
-  ::encode(laggy_since, bl);
-  ::encode(standby_for_rank, bl);
-  ::encode(standby_for_name, bl);
-  ::encode(export_targets, bl);
+  using ceph::encode;
+  encode(struct_v, bl);
+  encode(global_id, bl);
+  encode(name, bl);
+  encode(rank, bl);
+  encode(inc, bl);
+  encode((int32_t)state, bl);
+  encode(state_seq, bl);
+  encode(addrs.legacy_addr(), bl, 0);
+  encode(laggy_since, bl);
+  encode(MDS_RANK_NONE, bl);
+  encode(std::string(), bl);
+  encode(export_targets, bl);
 }
 
-void MDSMap::mds_info_t::decode(bufferlist::iterator& bl)
+void MDSMap::mds_info_t::decode(bufferlist::const_iterator& bl)
 {
-  DECODE_START_LEGACY_COMPAT_LEN(7, 4, 4, bl);
-  ::decode(global_id, bl);
-  ::decode(name, bl);
-  ::decode(rank, bl);
-  ::decode(inc, bl);
-  ::decode((int32_t&)(state), bl);
-  ::decode(state_seq, bl);
-  ::decode(addr, bl);
-  ::decode(laggy_since, bl);
-  ::decode(standby_for_rank, bl);
-  ::decode(standby_for_name, bl);
+  DECODE_START_LEGACY_COMPAT_LEN(9, 4, 4, bl);
+  decode(global_id, bl);
+  decode(name, bl);
+  decode(rank, bl);
+  decode(inc, bl);
+  decode((int32_t&)(state), bl);
+  decode(state_seq, bl);
+  decode(addrs, bl);
+  decode(laggy_since, bl);
+  {
+    mds_rank_t standby_for_rank;
+    decode(standby_for_rank, bl);
+  }
+  {
+    std::string standby_for_name;
+    decode(standby_for_name, bl);
+  }
   if (struct_v >= 2)
-    ::decode(export_targets, bl);
+    decode(export_targets, bl);
   if (struct_v >= 5)
-    ::decode(mds_features, bl);
+    decode(mds_features, bl);
   if (struct_v >= 6) {
-    ::decode(standby_for_fscid, bl);
+    fs_cluster_id_t standby_for_fscid;
+    decode(standby_for_fscid, bl);
   }
   if (struct_v >= 7) {
-    ::decode(standby_replay, bl);
+    bool standby_replay;
+    decode(standby_replay, bl);
+  }
+  if (struct_v >= 9) {
+    decode(flags, bl);
   }
   DECODE_FINISH(bl);
 }
@@ -533,111 +609,114 @@ void MDSMap::encode(bufferlist& bl, uint64_t features) const
     inc.insert(std::make_pair(rank, epoch));
   }
 
+  using ceph::encode;
   if ((features & CEPH_FEATURE_PGID64) == 0) {
     __u16 v = 2;
-    ::encode(v, bl);
-    ::encode(epoch, bl);
-    ::encode(flags, bl);
-    ::encode(last_failure, bl);
-    ::encode(root, bl);
-    ::encode(session_timeout, bl);
-    ::encode(session_autoclose, bl);
-    ::encode(max_file_size, bl);
-    ::encode(max_mds, bl);
+    encode(v, bl);
+    encode(epoch, bl);
+    encode(flags, bl);
+    encode(last_failure, bl);
+    encode(root, bl);
+    encode(session_timeout, bl);
+    encode(session_autoclose, bl);
+    encode(max_file_size, bl);
+    encode(max_mds, bl);
     __u32 n = mds_info.size();
-    ::encode(n, bl);
+    encode(n, bl);
     for (map<mds_gid_t, mds_info_t>::const_iterator i = mds_info.begin();
 	i != mds_info.end(); ++i) {
-      ::encode(i->first, bl);
-      ::encode(i->second, bl, features);
+      encode(i->first, bl);
+      encode(i->second, bl, features);
     }
     n = data_pools.size();
-    ::encode(n, bl);
+    encode(n, bl);
     for (const auto p: data_pools) {
       n = p;
-      ::encode(n, bl);
+      encode(n, bl);
     }
 
     int32_t m = cas_pool;
-    ::encode(m, bl);
+    encode(m, bl);
     return;
   } else if ((features & CEPH_FEATURE_MDSENC) == 0) {
     __u16 v = 3;
-    ::encode(v, bl);
-    ::encode(epoch, bl);
-    ::encode(flags, bl);
-    ::encode(last_failure, bl);
-    ::encode(root, bl);
-    ::encode(session_timeout, bl);
-    ::encode(session_autoclose, bl);
-    ::encode(max_file_size, bl);
-    ::encode(max_mds, bl);
+    encode(v, bl);
+    encode(epoch, bl);
+    encode(flags, bl);
+    encode(last_failure, bl);
+    encode(root, bl);
+    encode(session_timeout, bl);
+    encode(session_autoclose, bl);
+    encode(max_file_size, bl);
+    encode(max_mds, bl);
     __u32 n = mds_info.size();
-    ::encode(n, bl);
+    encode(n, bl);
     for (map<mds_gid_t, mds_info_t>::const_iterator i = mds_info.begin();
 	i != mds_info.end(); ++i) {
-      ::encode(i->first, bl);
-      ::encode(i->second, bl, features);
+      encode(i->first, bl);
+      encode(i->second, bl, features);
     }
-    ::encode(data_pools, bl);
-    ::encode(cas_pool, bl);
+    encode(data_pools, bl);
+    encode(cas_pool, bl);
 
     // kclient ignores everything from here
     __u16 ev = 5;
-    ::encode(ev, bl);
-    ::encode(compat, bl);
-    ::encode(metadata_pool, bl);
-    ::encode(created, bl);
-    ::encode(modified, bl);
-    ::encode(tableserver, bl);
-    ::encode(in, bl);
-    ::encode(inc, bl);
-    ::encode(up, bl);
-    ::encode(failed, bl);
-    ::encode(stopped, bl);
-    ::encode(last_failure_osd_epoch, bl);
+    encode(ev, bl);
+    encode(compat, bl);
+    encode(metadata_pool, bl);
+    encode(created, bl);
+    encode(modified, bl);
+    encode(tableserver, bl);
+    encode(in, bl);
+    encode(inc, bl);
+    encode(up, bl);
+    encode(failed, bl);
+    encode(stopped, bl);
+    encode(last_failure_osd_epoch, bl);
     return;
   }
 
   ENCODE_START(5, 4, bl);
-  ::encode(epoch, bl);
-  ::encode(flags, bl);
-  ::encode(last_failure, bl);
-  ::encode(root, bl);
-  ::encode(session_timeout, bl);
-  ::encode(session_autoclose, bl);
-  ::encode(max_file_size, bl);
-  ::encode(max_mds, bl);
-  ::encode(mds_info, bl, features);
-  ::encode(data_pools, bl);
-  ::encode(cas_pool, bl);
+  encode(epoch, bl);
+  encode(flags, bl);
+  encode(last_failure, bl);
+  encode(root, bl);
+  encode(session_timeout, bl);
+  encode(session_autoclose, bl);
+  encode(max_file_size, bl);
+  encode(max_mds, bl);
+  encode(mds_info, bl, features);
+  encode(data_pools, bl);
+  encode(cas_pool, bl);
 
   // kclient ignores everything from here
-  __u16 ev = 12;
-  ::encode(ev, bl);
-  ::encode(compat, bl);
-  ::encode(metadata_pool, bl);
-  ::encode(created, bl);
-  ::encode(modified, bl);
-  ::encode(tableserver, bl);
-  ::encode(in, bl);
-  ::encode(inc, bl);
-  ::encode(up, bl);
-  ::encode(failed, bl);
-  ::encode(stopped, bl);
-  ::encode(last_failure_osd_epoch, bl);
-  ::encode(ever_allowed_features, bl);
-  ::encode(explicitly_allowed_features, bl);
-  ::encode(inline_data_enabled, bl);
-  ::encode(enabled, bl);
-  ::encode(fs_name, bl);
-  ::encode(damaged, bl);
-  ::encode(balancer, bl);
-  ::encode(standby_count_wanted, bl);
+  __u16 ev = 14;
+  encode(ev, bl);
+  encode(compat, bl);
+  encode(metadata_pool, bl);
+  encode(created, bl);
+  encode(modified, bl);
+  encode(tableserver, bl);
+  encode(in, bl);
+  encode(inc, bl);
+  encode(up, bl);
+  encode(failed, bl);
+  encode(stopped, bl);
+  encode(last_failure_osd_epoch, bl);
+  encode(ever_allowed_features, bl);
+  encode(explicitly_allowed_features, bl);
+  encode(inline_data_enabled, bl);
+  encode(enabled, bl);
+  encode(fs_name, bl);
+  encode(damaged, bl);
+  encode(balancer, bl);
+  encode(standby_count_wanted, bl);
+  encode(old_max_mds, bl);
+  encode(min_compat_client, bl);
   ENCODE_FINISH(bl);
 }
 
-void MDSMap::sanitize(std::function<bool(int64_t pool)> pool_exists)
+void MDSMap::sanitize(const std::function<bool(int64_t pool)>& pool_exists)
 {
   /* Before we did stricter checking, it was possible to remove a data pool
    * without also deleting it from the MDSMap. Check for that here after
@@ -654,92 +733,85 @@ void MDSMap::sanitize(std::function<bool(int64_t pool)> pool_exists)
   }
 }
 
-void MDSMap::decode(bufferlist::iterator& p)
+void MDSMap::decode(bufferlist::const_iterator& p)
 {
   std::map<mds_rank_t,int32_t> inc;  // Legacy field, parse and drop
 
   cached_up_features = 0;
   DECODE_START_LEGACY_COMPAT_LEN_16(5, 4, 4, p);
-  ::decode(epoch, p);
-  ::decode(flags, p);
-  ::decode(last_failure, p);
-  ::decode(root, p);
-  ::decode(session_timeout, p);
-  ::decode(session_autoclose, p);
-  ::decode(max_file_size, p);
-  ::decode(max_mds, p);
-  ::decode(mds_info, p);
+  decode(epoch, p);
+  decode(flags, p);
+  decode(last_failure, p);
+  decode(root, p);
+  decode(session_timeout, p);
+  decode(session_autoclose, p);
+  decode(max_file_size, p);
+  decode(max_mds, p);
+  decode(mds_info, p);
   if (struct_v < 3) {
     __u32 n;
-    ::decode(n, p);
+    decode(n, p);
     while (n--) {
       __u32 m;
-      ::decode(m, p);
+      decode(m, p);
       data_pools.push_back(m);
     }
     __s32 s;
-    ::decode(s, p);
+    decode(s, p);
     cas_pool = s;
   } else {
-    ::decode(data_pools, p);
-    ::decode(cas_pool, p);
+    decode(data_pools, p);
+    decode(cas_pool, p);
   }
 
   // kclient ignores everything from here
   __u16 ev = 1;
   if (struct_v >= 2)
-    ::decode(ev, p);
+    decode(ev, p);
   if (ev >= 3)
-    ::decode(compat, p);
+    decode(compat, p);
   else
     compat = get_compat_set_base();
   if (ev < 5) {
     __u32 n;
-    ::decode(n, p);
+    decode(n, p);
     metadata_pool = n;
   } else {
-    ::decode(metadata_pool, p);
+    decode(metadata_pool, p);
   }
-  ::decode(created, p);
-  ::decode(modified, p);
-  ::decode(tableserver, p);
-  ::decode(in, p);
-  ::decode(inc, p);
-  ::decode(up, p);
-  ::decode(failed, p);
-  ::decode(stopped, p);
+  decode(created, p);
+  decode(modified, p);
+  decode(tableserver, p);
+  decode(in, p);
+  decode(inc, p);
+  decode(up, p);
+  decode(failed, p);
+  decode(stopped, p);
   if (ev >= 4)
-    ::decode(last_failure_osd_epoch, p);
+    decode(last_failure_osd_epoch, p);
   if (ev >= 6) {
     if (ev < 10) {
       // previously this was a bool about snaps, not a flag map
       bool flag;
-      ::decode(flag, p);
+      decode(flag, p);
       ever_allowed_features = flag ? CEPH_MDSMAP_ALLOW_SNAPS : 0;
-      ever_allowed_features |= CEPH_MDSMAP_ALLOW_MULTIMDS|CEPH_MDSMAP_ALLOW_DIRFRAGS;
-      ::decode(flag, p);
+      decode(flag, p);
       explicitly_allowed_features = flag ? CEPH_MDSMAP_ALLOW_SNAPS : 0;
-      if (max_mds > 1) {
-	set_multimds_allowed();
-      }
     } else {
-      ::decode(ever_allowed_features, p);
-      ::decode(explicitly_allowed_features, p);
+      decode(ever_allowed_features, p);
+      decode(explicitly_allowed_features, p);
     }
   } else {
-    ever_allowed_features = CEPH_MDSMAP_ALLOW_CLASSICS;
+    ever_allowed_features = 0;
     explicitly_allowed_features = 0;
-    if (max_mds > 1) {
-      set_multimds_allowed();
-    }
   }
   if (ev >= 7)
-    ::decode(inline_data_enabled, p);
+    decode(inline_data_enabled, p);
 
   if (ev >= 8) {
-    assert(struct_v >= 5);
-    ::decode(enabled, p);
-    ::decode(fs_name, p);
+    ceph_assert(struct_v >= 5);
+    decode(enabled, p);
+    decode(fs_name, p);
   } else {
     if (epoch > 1) {
       // If an MDS has ever been started, epoch will be greater than 1,
@@ -753,15 +825,23 @@ void MDSMap::decode(bufferlist::iterator& p)
   }
 
   if (ev >= 9) {
-    ::decode(damaged, p);
+    decode(damaged, p);
   }
 
   if (ev >= 11) {
-    ::decode(balancer, p);
+    decode(balancer, p);
   }
 
   if (ev >= 12) {
-    ::decode(standby_count_wanted, p);
+    decode(standby_count_wanted, p);
+  }
+
+  if (ev >= 13) {
+    decode(old_max_mds, p);
+  }
+
+  if (ev >= 14) {
+    decode(min_compat_client, p);
   }
 
   DECODE_FINISH(p);
