@@ -20,18 +20,20 @@
 #include "mds/FSMap.h"
 #include "include/ceph_features.h"
 
-class MFSMap : public Message {
- public:
+class MFSMap : public MessageInstance<MFSMap> {
+public:
+  friend factory;
+
   epoch_t epoch;
   bufferlist encoded;
 
   version_t get_epoch() const { return epoch; }
-  const FSMap & get_fsmap() {return fsmap;}
+  const FSMap& get_fsmap() const {return fsmap;}
 
   MFSMap() : 
-    Message(CEPH_MSG_FS_MAP), epoch(0) {}
+    MessageInstance(CEPH_MSG_FS_MAP), epoch(0) {}
   MFSMap(const uuid_d &f, const FSMap &fsmap_) :
-    Message(CEPH_MSG_FS_MAP), epoch(fsmap_.get_epoch())
+    MessageInstance(CEPH_MSG_FS_MAP), epoch(fsmap_.get_epoch())
   {
     fsmap = fsmap_;
   }
@@ -41,20 +43,21 @@ private:
   ~MFSMap() override {}
 
 public:
-  const char *get_type_name() const override { return "fsmap"; }
+  std::string_view get_type_name() const override { return "fsmap"; }
   void print(ostream& out) const override {
     out << "fsmap(e " << epoch << ")";
   }
 
   // marshalling
   void decode_payload() override {
-    bufferlist::iterator p = payload.begin();
-    ::decode(epoch, p);
-    ::decode(fsmap, p);
+    auto p = payload.cbegin();
+    decode(epoch, p);
+    decode(fsmap, p);
   }
   void encode_payload(uint64_t features) override {
-    ::encode(epoch, payload);
-    ::encode(fsmap, payload, features);
+    using ceph::encode;
+    encode(epoch, payload);
+    encode(fsmap, payload, features);
   }
 };
 
