@@ -20,17 +20,19 @@
 #include "include/types.h"
 
 
-class MMDSResolveAck : public Message {
- public:
+class MMDSResolveAck : public MessageInstance<MMDSResolveAck> {
+public:
+  friend factory;
+
   map<metareqid_t, bufferlist> commit;
   vector<metareqid_t> abort;
 
-  MMDSResolveAck() : Message(MSG_MDS_RESOLVEACK) {}
-private:
+protected:
+  MMDSResolveAck() : MessageInstance(MSG_MDS_RESOLVEACK) {}
   ~MMDSResolveAck() override {}
 
 public:
-  const char *get_type_name() const override { return "resolve_ack"; }
+  std::string_view get_type_name() const override { return "resolve_ack"; }
   /*void print(ostream& out) const {
     out << "resolve_ack.size()
 	<< "+" << ambiguous_imap.size()
@@ -46,13 +48,15 @@ public:
   }
 
   void encode_payload(uint64_t features) override {
-    ::encode(commit, payload);
-    ::encode(abort, payload);
+    using ceph::encode;
+    encode(commit, payload);
+    encode(abort, payload);
   }
   void decode_payload() override {
-    bufferlist::iterator p = payload.begin();
-    ::decode(commit, p);
-    ::decode(abort, p);
+    using ceph::decode;
+    auto p = payload.cbegin();
+    decode(commit, p);
+    decode(abort, p);
   }
 };
 
