@@ -23,7 +23,7 @@
 #include "common/errno.h"
 #include "include/stringify.h"
 
-#include "include/assert.h" // re-clobber assert()
+#include "include/ceph_assert.h" // re-clobber ceph_assert()
 #define dout_subsys ceph_subsys_mon
 #undef dout_prefix
 #define dout_prefix _prefix(_dout, mon, this)
@@ -173,7 +173,7 @@ void ConfigKeyService::store_delete_prefix(
 bool ConfigKeyService::service_dispatch(MonOpRequestRef op)
 {
   Message *m = op->get_req();
-  assert(m != NULL);
+  ceph_assert(m != NULL);
   dout(10) << __func__ << " " << *m << dendl;
 
   if (!in_quorum()) {
@@ -182,11 +182,11 @@ bool ConfigKeyService::service_dispatch(MonOpRequestRef op)
     return false;
   }
 
-  assert(m->get_type() == MSG_MON_COMMAND);
+  ceph_assert(m->get_type() == MSG_MON_COMMAND);
 
   MMonCommand *cmd = static_cast<MMonCommand*>(m);
 
-  assert(!cmd->cmd.empty());
+  ceph_assert(!cmd->cmd.empty());
 
   int ret = 0;
   stringstream ss;
@@ -206,7 +206,7 @@ bool ConfigKeyService::service_dispatch(MonOpRequestRef op)
   if (prefix == "config-key get") {
     ret = store_get(key, rdata);
     if (ret < 0) {
-      assert(!rdata.length());
+      ceph_assert(!rdata.length());
       ss << "error obtaining '" << key << "': " << cpp_strerror(ret);
       goto out;
     }
@@ -229,27 +229,20 @@ bool ConfigKeyService::service_dispatch(MonOpRequestRef op)
       // they specified '-i <file>'
       data = cmd->get_data();
     }
-    if (data.length() > (size_t) g_conf->mon_config_key_max_entry_size) {
+    if (data.length() > (size_t) g_conf()->mon_config_key_max_entry_size) {
       ret = -EFBIG; // File too large
       ss << "error: entry size limited to "
-         << g_conf->mon_config_key_max_entry_size << " bytes. "
+         << g_conf()->mon_config_key_max_entry_size << " bytes. "
          << "Use 'mon config key max entry size' to manually adjust";
       goto out;
     }
 
     std::string mgr_prefix = "mgr/";
-    std::string dashboard_prefix = "mgr/dashboard";
-    if ((key.size() >= mgr_prefix.size() &&
-         key.substr(0, mgr_prefix.size()) == mgr_prefix)
-        &&
-        (key.size() < dashboard_prefix.size() ||
-         key.substr(0, dashboard_prefix.size()) != dashboard_prefix)) {
-
+    if (key.size() >= mgr_prefix.size() &&
+        key.substr(0, mgr_prefix.size()) == mgr_prefix) {
       // In <= mimic, we used config-key for mgr module configuration,
       // and we bring values forward in an upgrade, but subsequent
-      // `set` operations will not be picked up.  Warn user about this,
-      // unless they're addressing the dashboard, which uses config-key
-      // for its certificates in Mimic (but not in Nautilus)
+      // `set` operations will not be picked up.  Warn user about this.
       ss << "WARNING: it looks like you might be trying to set a ceph-mgr "
             "module configuration key.  Since Ceph 13.0.0 (Mimic), mgr module "
             "configuration is done with `config set`, and new values "
@@ -382,7 +375,7 @@ void ConfigKeyService::do_osd_new(
     const uuid_d& uuid,
     const string& dmcrypt_key)
 {
-  assert(paxos->is_plugged());
+  ceph_assert(paxos->is_plugged());
 
   string dmcrypt_key_prefix = _get_dmcrypt_prefix(uuid, "luks");
   bufferlist dmcrypt_key_value;
