@@ -12,6 +12,9 @@
 #include "rgw_cr_rest.h"
 #include "rgw_op.h"
 #include "rgw_es_query.h"
+#include "rgw_zone.h"
+
+#include "services/svc_zone.h"
 
 #include "include/str_list.h"
 
@@ -169,28 +172,28 @@ struct ElasticConfig {
   uint32_t num_replicas{0};
   std::map <string,string> default_headers = {{ "Content-Type", "application/json" }};
 
-  void init(CephContext *cct, const map<string, string, ltstr_nocase>& config) {
-    string elastic_endpoint = rgw_conf_get(config, "endpoint", "");
+  void init(CephContext *cct, const JSONFormattable& config) {
+    string elastic_endpoint = config["endpoint"];
     id = string("elastic:") + elastic_endpoint;
     conn.reset(new RGWRESTConn(cct, nullptr, id, { elastic_endpoint }));
-    explicit_custom_meta = rgw_conf_get_bool(config, "explicit_custom_meta", true);
-    index_buckets.init(rgw_conf_get(config, "index_buckets_list", ""), true); /* approve all buckets by default */
-    allow_owners.init(rgw_conf_get(config, "approved_owners_list", ""), true); /* approve all bucket owners by default */
-    override_index_path = rgw_conf_get(config, "override_index_path", "");
-    num_shards = rgw_conf_get_int(config, "num_shards", ES_NUM_SHARDS_DEFAULT);
+    explicit_custom_meta = config["explicit_custom_meta"](true);
+    index_buckets.init(config["index_buckets_list"], true); /* approve all buckets by default */
+    allow_owners.init(config["approved_owners_list"], true); /* approve all bucket owners by default */
+    override_index_path = config["override_index_path"];
+    num_shards = config["num_shards"](ES_NUM_SHARDS_DEFAULT);
     if (num_shards < ES_NUM_SHARDS_MIN) {
       num_shards = ES_NUM_SHARDS_MIN;
     }
-    num_replicas = rgw_conf_get_int(config, "num_replicas", ES_NUM_REPLICAS_DEFAULT);
-    string user = rgw_conf_get(config, "username", "");
-    string pw = rgw_conf_get(config, "password", "");
-    if (!user.empty() && !pw.empty()) {
+    num_replicas = config["num_replicas"](ES_NUM_REPLICAS_DEFAULT);
+    if (string user = config["username"], pw = config["password"];
+        !user.empty() && !pw.empty()) {
       auto auth_string = user + ":" + pw;
       default_headers.emplace("AUTHORIZATION", "Basic " + rgw::to_base64(auth_string));
     }
+
   }
 
-  void init_instance(RGWRealm& realm, uint64_t instance_id) {
+  void init_instance(const RGWRealm& realm, uint64_t instance_id) {
     sync_instance = instance_id;
 
     if (!override_index_path.empty()) {
@@ -253,7 +256,7 @@ static const char *es_type_to_str(const ESType& t) {
 struct es_type_v2 {
   ESType estype;
   const char *format{nullptr};
-  boost::optional<bool> analyzed;
+  std::optional<bool> analyzed;
 
   es_type_v2(ESType et) : estype(et) {}
 
@@ -280,8 +283,8 @@ struct es_type_v2 {
 struct es_type_v5 {
   ESType estype;
   const char *format{nullptr};
-  boost::optional<bool> analyzed;
-  boost::optional<bool> index;
+  std::optional<bool> analyzed;
+  std::optional<bool> index;
 
   es_type_v5(ESType et) : estype(et) {}
 
@@ -402,13 +405,14 @@ struct es_index_config : public es_index_config_base {
 };
 
 static bool is_sys_attr(const std::string& attr_name){
-  static constexpr std::initializer_list<const char*> rgw_sys_attrs = {RGW_ATTR_PG_VER,
-                                                                       RGW_ATTR_SOURCE_ZONE,
-                                                                       RGW_ATTR_ID_TAG,
-                                                                       RGW_ATTR_TEMPURL_KEY1,
-                                                                       RGW_ATTR_TEMPURL_KEY2,
-                                                                       RGW_ATTR_UNIX1,
-                                                                       RGW_ATTR_UNIX_KEY1
+  static constexpr std::initializer_list<const char*> rgw_sys_attrs =
+                                                         {RGW_ATTR_PG_VER,
+                                                          RGW_ATTR_SOURCE_ZONE,
+                                                          RGW_ATTR_ID_TAG,
+                                                          RGW_ATTR_TEMPURL_KEY1,
+                                                          RGW_ATTR_TEMPURL_KEY2,
+                                                          RGW_ATTR_UNIX1,
+                                                          RGW_ATTR_UNIX_KEY1
   };
 
   return std::find(rgw_sys_attrs.begin(), rgw_sys_attrs.end(), attr_name) != rgw_sys_attrs.end();
@@ -471,8 +475,8 @@ struct es_obj_metadata {
 
       if (attr_name == RGW_ATTR_ACL) {
         try {
-          auto i = val.begin();
-          ::decode(policy, i);
+          auto i = val.cbegin();
+          decode(policy, i);
         } catch (buffer::error& err) {
           ldout(cct, 0) << "ERROR: failed to decode acl for " << bucket_info.bucket << "/" << key << dendl;
           continue;
@@ -493,8 +497,8 @@ struct es_obj_metadata {
         }
       } else if (attr_name == RGW_ATTR_TAGS) {
         try {
-          auto tags_bl = val.begin();
-          ::decode(obj_tags, tags_bl);
+          auto tags_bl = val.cbegin();
+          decode(obj_tags, tags_bl);
         } catch (buffer::error& err) {
           ldout(cct,0) << "ERROR: failed to decode obj tags for "
                        << bucket_info.bucket << "/" << key << dendl;
@@ -503,8 +507,8 @@ struct es_obj_metadata {
       } else if (attr_name == RGW_ATTR_COMPRESSION) {
         RGWCompressionInfo cs_info;
         try {
-          auto vals_bl = val.begin();
-          ::decode(cs_info, vals_bl);
+          auto vals_bl = val.cbegin();
+          decode(cs_info, vals_bl);
         } catch (buffer::error& err) {
           ldout(cct,0) << "ERROR: failed to decode compression attr for "
                        << bucket_info.bucket << "/" << key << dendl;
@@ -790,20 +794,20 @@ public:
 class RGWElasticDataSyncModule : public RGWDataSyncModule {
   ElasticConfigRef conf;
 public:
-  RGWElasticDataSyncModule(CephContext *cct, const map<string, string, ltstr_nocase>& config) : conf(std::make_shared<ElasticConfig>()) {
+  RGWElasticDataSyncModule(CephContext *cct, const JSONFormattable& config) : conf(std::make_shared<ElasticConfig>()) {
     conf->init(cct, config);
   }
   ~RGWElasticDataSyncModule() override {}
 
   void init(RGWDataSyncEnv *sync_env, uint64_t instance_id) override {
-    conf->init_instance(sync_env->store->get_realm(), instance_id);
+    conf->init_instance(sync_env->store->svc.zone->get_realm(), instance_id);
   }
 
   RGWCoroutine *init_sync(RGWDataSyncEnv *sync_env) override {
     ldout(sync_env->cct, 5) << conf->id << ": init" << dendl;
     return new RGWElasticInitConfigCBCR(sync_env, conf);
   }
-  RGWCoroutine *sync_object(RGWDataSyncEnv *sync_env, RGWBucketInfo& bucket_info, rgw_obj_key& key, boost::optional<uint64_t> versioned_epoch, rgw_zone_set *zones_trace) override {
+  RGWCoroutine *sync_object(RGWDataSyncEnv *sync_env, RGWBucketInfo& bucket_info, rgw_obj_key& key, std::optional<uint64_t> versioned_epoch, rgw_zone_set *zones_trace) override {
     ldout(sync_env->cct, 10) << conf->id << ": sync_object: b=" << bucket_info.bucket << " k=" << key << " versioned_epoch=" << versioned_epoch.value_or(0) << dendl;
     if (!conf->should_handle_operation(bucket_info)) {
       ldout(sync_env->cct, 10) << conf->id << ": skipping operation (bucket not approved)" << dendl;
@@ -840,7 +844,7 @@ public:
   }
 };
 
-RGWElasticSyncModuleInstance::RGWElasticSyncModuleInstance(CephContext *cct, const map<string, string, ltstr_nocase>& config)
+RGWElasticSyncModuleInstance::RGWElasticSyncModuleInstance(CephContext *cct, const JSONFormattable& config)
 {
   data_handler = std::unique_ptr<RGWElasticDataSyncModule>(new RGWElasticDataSyncModule(cct, config));
 }
@@ -871,12 +875,8 @@ RGWRESTMgr *RGWElasticSyncModuleInstance::get_rest_filter(int dialect, RGWRESTMg
   return new RGWRESTMgr_MDSearch_S3();
 }
 
-int RGWElasticSyncModule::create_instance(CephContext *cct, map<string, string, ltstr_nocase>& config, RGWSyncModuleInstanceRef *instance) {
-  string endpoint;
-  auto i = config.find("endpoint");
-  if (i != config.end()) {
-    endpoint = i->second;
-  }
+int RGWElasticSyncModule::create_instance(CephContext *cct, const JSONFormattable& config, RGWSyncModuleInstanceRef *instance) {
+  string endpoint = config["endpoint"];
   instance->reset(new RGWElasticSyncModuleInstance(cct, config));
   return 0;
 }
