@@ -14,6 +14,7 @@
 #include "test/librbd/mock/MockOperations.h"
 #include "test/librbd/mock/MockReadahead.h"
 #include "test/librbd/mock/io/MockImageRequestWQ.h"
+#include "test/librbd/mock/io/MockObjectDispatcher.h"
 #include "common/RWLock.h"
 #include "common/WorkQueue.h"
 #include "common/zipkin_trace.h"
@@ -34,7 +35,7 @@ struct MockImageCtx {
                               const std::string &image_id,
                               const char *snap, librados::IoCtx& p,
                               bool read_only) {
-    assert(s_instance != nullptr);
+    ceph_assert(s_instance != nullptr);
     return s_instance;
   }
   MOCK_METHOD0(destroy, void());
@@ -51,8 +52,6 @@ struct MockImageCtx {
       snaps(image_ctx.snaps),
       snap_info(image_ctx.snap_info),
       snap_ids(image_ctx.snap_ids),
-      object_cacher(image_ctx.object_cacher),
-      object_set(image_ctx.object_set),
       old_format(image_ctx.old_format),
       read_only(image_ctx.read_only),
       clone_copy_on_read(image_ctx.clone_copy_on_read),
@@ -61,8 +60,8 @@ struct MockImageCtx {
       lock_tag(image_ctx.lock_tag),
       owner_lock(image_ctx.owner_lock),
       md_lock(image_ctx.md_lock),
-      cache_lock(image_ctx.cache_lock),
       snap_lock(image_ctx.snap_lock),
+      timestamp_lock(image_ctx.timestamp_lock),
       parent_lock(image_ctx.parent_lock),
       object_map_lock(image_ctx.object_map_lock),
       async_ops_lock(image_ctx.async_ops_lock),
@@ -71,6 +70,8 @@ struct MockImageCtx {
       size(image_ctx.size),
       features(image_ctx.features),
       flags(image_ctx.flags),
+      op_features(image_ctx.op_features),
+      operations_disabled(image_ctx.operations_disabled),
       stripe_unit(image_ctx.stripe_unit),
       stripe_count(image_ctx.stripe_count),
       object_prefix(image_ctx.object_prefix),
@@ -82,6 +83,7 @@ struct MockImageCtx {
       group_spec(image_ctx.group_spec),
       layout(image_ctx.layout),
       io_work_queue(new io::MockImageRequestWQ()),
+      io_object_dispatcher(new io::MockObjectDispatcher()),
       op_work_queue(new MockContextWQ()),
       readahead_max_bytes(image_ctx.readahead_max_bytes),
       event_socket(image_ctx.event_socket),
@@ -90,26 +92,17 @@ struct MockImageCtx {
       image_watcher(NULL), object_map(NULL),
       exclusive_lock(NULL), journal(NULL),
       trace_endpoint(image_ctx.trace_endpoint),
-      concurrent_management_ops(image_ctx.concurrent_management_ops),
-      blacklist_on_break_lock(image_ctx.blacklist_on_break_lock),
-      blacklist_expire_seconds(image_ctx.blacklist_expire_seconds),
       sparse_read_threshold_bytes(image_ctx.sparse_read_threshold_bytes),
-      journal_order(image_ctx.journal_order),
-      journal_splay_width(image_ctx.journal_splay_width),
-      journal_commit_age(image_ctx.journal_commit_age),
-      journal_object_flush_interval(image_ctx.journal_object_flush_interval),
-      journal_object_flush_bytes(image_ctx.journal_object_flush_bytes),
-      journal_object_flush_age(image_ctx.journal_object_flush_age),
-      journal_pool(image_ctx.journal_pool),
-      journal_max_payload_bytes(image_ctx.journal_max_payload_bytes),
-      journal_max_concurrent_object_sets(
-          image_ctx.journal_max_concurrent_object_sets),
-      mirroring_resync_after_disconnect(
-          image_ctx.mirroring_resync_after_disconnect),
+      discard_granularity_bytes(image_ctx.discard_granularity_bytes),
       mirroring_replay_delay(image_ctx.mirroring_replay_delay),
       non_blocking_aio(image_ctx.non_blocking_aio),
       blkin_trace_all(image_ctx.blkin_trace_all),
-      enable_alloc_hint(image_ctx.enable_alloc_hint)
+      enable_alloc_hint(image_ctx.enable_alloc_hint),
+      ignore_migrating(image_ctx.ignore_migrating),
+      mtime_update_interval(image_ctx.mtime_update_interval),
+      atime_update_interval(image_ctx.atime_update_interval),
+      cache(image_ctx.cache),
+      config(image_ctx.config)
   {
     md_ctx.dup(image_ctx.md_ctx);
     data_ctx.dup(image_ctx.data_ctx);
@@ -129,6 +122,7 @@ struct MockImageCtx {
     delete image_watcher;
     delete op_work_queue;
     delete io_work_queue;
+    delete io_object_dispatcher;
   }
 
   void wait_for_async_requests() {
@@ -153,14 +147,18 @@ struct MockImageCtx {
   MOCK_CONST_METHOD1(get_image_size, uint64_t(librados::snap_t));
   MOCK_CONST_METHOD1(get_object_count, uint64_t(librados::snap_t));
   MOCK_CONST_METHOD1(get_read_flags, int(librados::snap_t));
+  MOCK_CONST_METHOD2(get_flags, int(librados::snap_t in_snap_id,
+                     uint64_t *flags));
   MOCK_CONST_METHOD2(get_snap_id,
 		     librados::snap_t(cls::rbd::SnapshotNamespace snap_namespace,
 				      std::string in_snap_name));
   MOCK_CONST_METHOD1(get_snap_info, const SnapInfo*(librados::snap_t));
+  MOCK_CONST_METHOD2(get_snap_name, int(librados::snap_t, std::string *));
   MOCK_CONST_METHOD2(get_snap_namespace, int(librados::snap_t,
 					     cls::rbd::SnapshotNamespace *out_snap_namespace));
   MOCK_CONST_METHOD2(get_parent_spec, int(librados::snap_t in_snap_id,
-                                          ParentSpec *pspec));
+                                          cls::rbd::ParentImageSpec *pspec));
+  MOCK_CONST_METHOD1(get_parent_info, const ParentImageInfo*(librados::snap_t));
   MOCK_CONST_METHOD2(get_parent_overlap, int(librados::snap_t in_snap_id,
                                              uint64_t *overlap));
   MOCK_CONST_METHOD2(prune_parent_extents, uint64_t(vector<pair<uint64_t,uint64_t> >& ,
@@ -171,29 +169,32 @@ struct MockImageCtx {
   MOCK_CONST_METHOD2(is_snap_unprotected, int(librados::snap_t in_snap_id,
                                               bool *is_unprotected));
 
+  MOCK_CONST_METHOD0(get_create_timestamp, utime_t());
+  MOCK_CONST_METHOD0(get_access_timestamp, utime_t());
+  MOCK_CONST_METHOD0(get_modify_timestamp, utime_t());
+
+  MOCK_METHOD1(set_access_timestamp, void(const utime_t at));
+  MOCK_METHOD1(set_modify_timestamp, void(const utime_t at));
+
   MOCK_METHOD8(add_snap, void(cls::rbd::SnapshotNamespace in_snap_namespace,
 			      std::string in_snap_name,
 			      librados::snap_t id,
-			      uint64_t in_size, const ParentInfo &parent,
+			      uint64_t in_size, const ParentImageInfo &parent,
 			      uint8_t protection_status, uint64_t flags, utime_t timestamp));
   MOCK_METHOD3(rm_snap, void(cls::rbd::SnapshotNamespace in_snap_namespace,
 			     std::string in_snap_name,
 			     librados::snap_t id));
 
   MOCK_METHOD0(user_flushed, void());
-  MOCK_METHOD1(flush, void(Context *));
   MOCK_METHOD1(flush_async_operations, void(Context *));
   MOCK_METHOD1(flush_copyup, void(Context *));
 
-  MOCK_METHOD1(flush_cache, void(Context *));
-  MOCK_METHOD2(invalidate_cache, void(bool, Context *));
-  MOCK_METHOD1(shut_down_cache, void(Context *));
-  MOCK_METHOD0(is_cache_empty, bool());
-
   MOCK_CONST_METHOD1(test_features, bool(uint64_t test_features));
   MOCK_CONST_METHOD2(test_features, bool(uint64_t test_features,
                                          const RWLock &in_snap_lock));
 
+  MOCK_CONST_METHOD1(test_op_features, bool(uint64_t op_features));
+
   MOCK_METHOD1(cancel_async_requests, void(Context*));
 
   MOCK_METHOD0(create_exclusive_lock, MockExclusiveLock*());
@@ -204,14 +205,9 @@ struct MockImageCtx {
   MOCK_METHOD1(notify_update, void(Context *));
 
   MOCK_CONST_METHOD0(get_exclusive_lock_policy, exclusive_lock::Policy*());
-
   MOCK_CONST_METHOD0(get_journal_policy, journal::Policy*());
-  MOCK_CONST_METHOD1(set_journal_policy, void(journal::Policy*));
+  MOCK_METHOD1(set_journal_policy, void(journal::Policy*));
 
-  MOCK_METHOD8(aio_read_from_cache, void(object_t, uint64_t, bufferlist *,
-                                         size_t, uint64_t, Context *, int, ZTracer::Trace *));
-  MOCK_METHOD8(write_to_cache, void(object_t, const bufferlist&, size_t,
-                                    uint64_t, Context *, int, uint64_t, ZTracer::Trace *));
   MOCK_METHOD2(apply_metadata, int(const std::map<std::string, bufferlist> &,
                                    bool));
 
@@ -234,9 +230,6 @@ struct MockImageCtx {
   std::map<librados::snap_t, SnapInfo> snap_info;
   std::map<std::pair<cls::rbd::SnapshotNamespace, std::string>, librados::snap_t> snap_ids;
 
-  ObjectCacher *object_cacher;
-  ObjectCacher::ObjectSet *object_set;
-
   bool old_format;
   bool read_only;
 
@@ -252,8 +245,8 @@ struct MockImageCtx {
 
   RWLock &owner_lock;
   RWLock &md_lock;
-  Mutex &cache_lock;
   RWLock &snap_lock;
+  RWLock &timestamp_lock;
   RWLock &parent_lock;
   RWLock &object_map_lock;
   Mutex &async_ops_lock;
@@ -263,13 +256,16 @@ struct MockImageCtx {
   uint64_t size;
   uint64_t features;
   uint64_t flags;
+  uint64_t op_features;
+  bool operations_disabled;
   uint64_t stripe_unit;
   uint64_t stripe_count;
   std::string object_prefix;
   std::string header_oid;
   std::string id;
   std::string name;
-  ParentInfo parent_md;
+  ParentImageInfo parent_md;
+  MigrationInfo migration_info;
   char *format_string;
   cls::rbd::GroupSpec group_spec;
 
@@ -282,6 +278,7 @@ struct MockImageCtx {
   std::map<uint64_t, io::CopyupRequest<MockImageCtx>*> copyup_list;
 
   io::MockImageRequestWQ *io_work_queue;
+  io::MockObjectDispatcher *io_object_dispatcher;
   MockContextWQ *op_work_queue;
 
   cache::MockImageCache *image_cache = nullptr;
@@ -302,24 +299,18 @@ struct MockImageCtx {
 
   ZTracer::Endpoint trace_endpoint;
 
-  int concurrent_management_ops;
-  bool blacklist_on_break_lock;
-  uint32_t blacklist_expire_seconds;
   uint64_t sparse_read_threshold_bytes;
-  uint8_t journal_order;
-  uint8_t journal_splay_width;
-  double journal_commit_age;
-  int journal_object_flush_interval;
-  uint64_t journal_object_flush_bytes;
-  double journal_object_flush_age;
-  std::string journal_pool;
-  uint32_t journal_max_payload_bytes;
-  int journal_max_concurrent_object_sets;
-  bool mirroring_resync_after_disconnect;
+  uint32_t discard_granularity_bytes;
   int mirroring_replay_delay;
   bool non_blocking_aio;
   bool blkin_trace_all;
   bool enable_alloc_hint;
+  bool ignore_migrating;
+  uint64_t mtime_update_interval;
+  uint64_t atime_update_interval;
+  bool cache;
+
+  ConfigProxy config;
 };
 
 } // namespace librbd
